from ...Classes.Arc import Arc
from ...Classes.Arc2 import Arc2
from ...Classes.MachineSIPMSM import MachineSIPMSM

import sys
import gmsh
import cmath


def _find_point_tag(d={}, p=complex(0.0, 0.0)):
    """Find a point in the GMSH dictionary

    Parameters
    ----------
    d : Dictionary
        GMSH dictionary
    p : Complex
        Point coordinates

    Returns
    -------
    tag : int
        Existing tag number or new one if it does not exist
    real : float
        Real coordinates of point
    imag : float
        Imaginary coordinates of point
    """
    tol = 1e-6
    for s_id, s_data in d.items():
        for lid, lvalues in s_data.items():
            if type(lvalues) is not dict:
                continue
            for pid, pvalues in lvalues.items():
                if type(pvalues) is not dict:
                    continue
                if pvalues["tag"] is not None:
                    b = pvalues["coord"]
                    if abs(p.real - b.real) < tol and abs(p.imag - b.imag) < tol:
                        return pvalues["tag"], b.real, b.imag
    return None, p.real, p.imag


def _find_points_from_line(d={}, ltag=-1):
    """Find points tag from existing lines

    Parameters
    ----------
    d : Dictionary
        GMSH dictionary
    ltag : int
        line tag

    Returns
    -------
    coord : float
        Coordinates of point if found
    """
    btag = None
    etag = None
    ctag = None
    for s_id, s_data in d.items():
        for lid, lvalues in s_data.items():
            if type(lvalues) is not dict:
                continue
            if lvalues["tag"] != ltag:
                continue
            else:
                for pid, pvalues in lvalues.items():
                    if type(pvalues) is not dict:
                        continue
                    if pid == "begin":
                        btag = pvalues["tag"]
                    elif pid == "end":
                        etag = pvalues["tag"]
                    elif pid == "cent":
                        ctag = pvalues["tag"]
                break
    return [btag, etag, ctag]


def _find_lines_from_point(d={}, ptag=-1):
    """Find lines that have the given point tag

    Parameters
    ----------
    d : Dictionary
        GMSH dictionary
    ptag : int
        point tag

    Returns
    -------
    ltag : int
        List of line tags
    """
    lines = list()
    for s_id, s_data in d.items():
        for lid, lvalues in s_data.items():
            if type(lvalues) is not dict:
                continue
            for pid, pvalues in lvalues.items():
                if type(pvalues) is not dict:
                    continue
                if pvalues["tag"] == ptag:
                    lines.append(lvalues["tag"])
    return lines


def _add_line_to_dict(geo, line, d={}, idx=0, mesh_size=1e-2, n_elements=0):
    """Draw a new line and add it to GMSH dictionary if it does not exist

    Parameters
    ----------
    geo : Model
        GMSH Model objet
    line : Object
        Line Object
    d : Dictionary
        GMSH dictionary
    idx : int
        Surface index it belongs to
    mesh_size : float
        Points mesh size
    n_elements : int
        Number of elements on the line for meshing control

    Returns
    -------
    None
    """

    dlines = list()
    ltag = None
    btag, bx, by = _find_point_tag(d, line.get_begin())
    etag, ex, ey = _find_point_tag(d, line.get_end())
    if btag is None:
        btag = geo.addPoint(bx, by, 0, meshSize=mesh_size, tag=-1)
    else:
        dlines.extend(_find_lines_from_point(d, btag))
    if etag is None:
        etag = geo.addPoint(ex, ey, 0, meshSize=mesh_size, tag=-1)
    else:
        dlines.extend(_find_lines_from_point(d, etag))
    if isinstance(line, Arc):
        ctag, cx, cy = _find_point_tag(d, line.get_center())
        if ctag is None:
            ctag = geo.addPoint(cx, cy, 0, meshSize=mesh_size, tag=-1)
        else:
            dlines.extend(_find_lines_from_point(d, ctag))
        if len(dlines) > 0:
            for iline in dlines:
                p = _find_points_from_line(d, iline)
                if p[0] == btag and p[1] == etag and p[2] == ctag:
                    ltag = iline
                    break
                elif p[0] == etag and p[1] == btag and p[2] == ctag:
                    ltag = -iline
                    break
                else:
                    pass
            if ltag is None:
                ltag = geo.addCircleArc(btag, ctag, etag, tag=-1)
                if n_elements > 0:
                    geo.mesh.setTransfiniteCurve(ltag, n_elements + 1, "Progression")
        else:
            ltag = geo.addCircleArc(btag, ctag, etag, tag=-1)
            if n_elements > 0:
                geo.mesh.setTransfiniteCurve(ltag, n_elements + 1, "Progression")

        # To avoid fill the dictionary with repeated lines
        repeated = False
        for lid, lvalues in d[idx].items():
            if type(lvalues) is not dict:
                continue
            else:
                if lvalues["tag"] == ltag:
                    repeated = True

        if not repeated:
            nline = len(d[idx]) - 2
            d[idx].update(
                {
                    nline: {
                        "tag": ltag,
                        "n_elements": n_elements,
                        "begin": {"tag": btag, "coord": complex(bx, by)},
                        "end": {"tag": etag, "coord": complex(ex, ey)},
                        "cent": {"tag": ctag, "coord": complex(cx, cy)},
                    }
                }
            )

    else:
        if len(dlines) > 0:
            for iline in dlines:
                p = _find_points_from_line(d, iline)
                if p[0] == btag and p[1] == etag:
                    ltag = iline
                    break
                elif p[0] == etag and p[1] == btag:
                    ltag = -iline
                    break
                else:
                    pass
            if ltag is None:
                ltag = geo.addLine(btag, etag, tag=-1)
                if n_elements > 0:
                    geo.mesh.setTransfiniteCurve(ltag, n_elements + 1, "Progression")
        else:
            ltag = geo.addLine(btag, etag, tag=-1)
            if n_elements > 0:
                geo.mesh.setTransfiniteCurve(ltag, n_elements + 1, "Progression")

        # To avoid fill the dictionary with repeated lines
        repeated = False
        for lid, lvalues in d[idx].items():
            if type(lvalues) is not dict:
                continue
            else:
                if lvalues["tag"] == ltag:
                    repeated = True

        if not repeated:
            nline = len(d[idx]) - 2
            d[idx].update(
                {
                    nline: {
                        "tag": ltag,
                        "n_elements": n_elements,
                        "begin": {"tag": btag, "coord": complex(bx, by)},
                        "end": {"tag": etag, "coord": complex(ex, ey)},
                    }
                }
            )

    return None


def draw_GMSH(
    output,
    sym,
    is_antiper=False,
    is_remove_vent=False,
    is_remove_slotS=False,
    is_remove_slotR=False,
    is_lam_only_S=False,
    is_lam_only_R=False,
    kgeo_fineness=1,
    kmesh_fineness=1,
    user_mesh_dict={},
    path_save="GMSH_model.msh",
    is_sliding_band=True,
    transform_list=[],
):
    """Draws a machine mesh in GMSH format

    Parameters
    ----------
    output : Output
        Output object
    is_remove_vent : bool
        True to remove the ventilation ducts (Default value = False)
    is_remove_slotS : bool
        True to solve without slot effect on the Stator (Default value = False)
    is_remove_slotR : bool
        True to solve without slot effect on the Rotor (Default value = False)
    kgeo_fineness : float
        global coefficient to adjust geometry fineness
    kmesh_fineness : float
        global coefficient to adjust mesh fineness
    sym : int
        the symmetry applied on the stator and the rotor (take into account antiperiodicity)
    is_antiper: bool
        To apply antiperiodicity boundary conditions
    is_lam_only_S: bool
        Draw only stator lamination
    is_lam_only_R: bool
        Draw only rotor lamination

    Returns
    -------
    GMSH_dict : dict
        Dictionnary containing the main parameters of GMSH File
    """

    machine = output.simu.machine
    mesh_dict = {}
    tol = 1e-6
    # For readibility
    model = gmsh.model
    factory = model.geo

    # Start a new model
    gmsh.initialize(sys.argv)
    gmsh.option.setNumber("General.Terminal", int(True))
    gmsh.option.setNumber("Geometry.CopyMeshingMethod", 1)
    gmsh.option.setNumber("Geometry.PointNumbers", 0)
    gmsh.option.setNumber("Geometry.LineNumbers", 0)
    model.add("Pyleecan")

    rotor_list = list()
    rotor_list.extend(machine.shaft.build_geometry(sym=sym))
    rotor_list.extend(machine.rotor.build_geometry(sym=sym))
    stator_list = list()
    stator_list.extend(machine.stator.build_geometry(sym=sym))
    oo = factory.addPoint(0, 0, 0, 0, tag=-1)

    gmsh_dict = {
        0: {
            "tag": 0,
            "label": "origin",
            "with_holes": False,
            1: {
                "tag": 0,
                "n_elements": 1,
                "begin": {"tag": oo, "coord": complex(0.0, 0.0)},
                "end": {"tag": None, "coord": None},
                "cent": {"tag": None, "coord": None},
            },
        }
    }

    # Default rotor mesh element size
    mesh_size = machine.rotor.Rext / 25.0
    nsurf = 0
    for surf in rotor_list:
        nsurf += 1
        gmsh_dict.update({nsurf: {"tag": None, "label": surf.label}})
        if surf.label.find("Lamination_Rotor") != -1:
            gmsh_dict[nsurf]["with_holes"] = True
            lam_rotor_surf_id = nsurf
        else:
            gmsh_dict[nsurf]["with_holes"] = False
        if user_mesh_dict is not None:
            mesh_dict = surf.comp_mesh_dict(element_size=mesh_size)
            mesh_dict.update(user_mesh_dict)
        for line in surf.get_lines():
            # When symmetry is 1 the shaft surface is substrtacted from Rotor Lam instead
            if sym == 1 and line.label == "Lamination_Rotor_Yoke_Radius_Int":
                continue
            n_elem = mesh_dict.get(line.label)
            if (
                isinstance(line, Arc)
                and abs(line.get_angle() * 180.0 / cmath.pi) >= 180.0
            ):
                if line.is_trigo_direction == True:
                    arc1 = Arc2(
                        begin=line.get_begin(),
                        center=line.get_center(),
                        angle=cmath.pi / 2.0,
                        label=line.label,
                    )
                    arc2 = Arc2(
                        begin=arc1.get_end(),
                        center=line.get_center(),
                        angle=cmath.pi / 2.0,
                        label=line.label,
                    )
                else:
                    arc1 = Arc2(
                        begin=line.get_begin(),
                        center=line.get_center(),
                        angle=-cmath.pi / 2.0,
                        label=line.label,
                    )
                    arc2 = Arc2(
                        begin=arc1.get_end(),
                        center=line.get_center(),
                        angle=-cmath.pi / 2.0,
                        label=line.label,
                    )
                if n_elem is not None:
                    _add_line_to_dict(
                        geo=factory,
                        line=arc1,
                        d=gmsh_dict,
                        idx=nsurf,
                        n_elements=n_elem,
                    )
                    _add_line_to_dict(
                        geo=factory,
                        line=arc2,
                        d=gmsh_dict,
                        idx=nsurf,
                        n_elements=n_elem,
                    )
                else:
                    _add_line_to_dict(
                        geo=factory,
                        line=arc1,
                        d=gmsh_dict,
                        idx=nsurf,
                        mesh_size=mesh_size,
                    )
                    _add_line_to_dict(
                        geo=factory,
                        line=arc2,
                        d=gmsh_dict,
                        idx=nsurf,
                        mesh_size=mesh_size,
                    )
            elif isinstance(line, Arc) and (
                abs(line.get_angle() * 180.0 / cmath.pi) <= tol
            ):
                # Don't draw anything, this is a circle and usually is repeated ?
                pass
            else:
                if n_elem is not None:
                    _add_line_to_dict(
                        geo=factory,
                        line=line,
                        d=gmsh_dict,
                        idx=nsurf,
                        n_elements=n_elem,
                    )
                else:
                    _add_line_to_dict(
                        geo=factory,
                        line=line,
                        d=gmsh_dict,
                        idx=nsurf,
                        mesh_size=mesh_size,
                    )

    lam_and_holes = list()
    ext_lam_loop = None
    for s_id, s_data in gmsh_dict.items():
        lloop = []
        if s_id == 0:
            continue
        for lid, lvalues in s_data.items():
            if type(lvalues) is not dict:
                continue
            lloop.extend([lvalues["tag"]])
        cloop = factory.addCurveLoop(lloop)
        # search for the holes to substract from rotor lam
        if s_data["label"].find("Lamination_Rotor") != -1:
            ext_lam_loop = cloop
        else:
            # MachineSIPSM does not have holes in rotor lam
            # only shaft is taken out if symmetry is one
            if isinstance(machine, MachineSIPMSM):
                if sym == 1 and s_data["label"] == "Shaft":
                    lam_and_holes.extend([cloop])
            else:
                if sym == 1:
                    lam_and_holes.extend([cloop])
                elif s_data["label"] != "Shaft":
                    lam_and_holes.extend([cloop])
                else:
                    pass

            # Shaft, magnets and magnet pocket surfaces are created
            if not is_lam_only_R:
                s_data["tag"] = factory.addPlaneSurface([cloop], tag=-1)
                pg = model.addPhysicalGroup(2, [s_data["tag"]])
                model.setPhysicalName(2, pg, s_data["label"])

    # Finally rotor lamination is built
    if ext_lam_loop is not None:
        lam_and_holes.insert(0, ext_lam_loop)
    gmsh_dict[lam_rotor_surf_id]["tag"] = factory.addPlaneSurface(lam_and_holes, tag=-1)
    pg = model.addPhysicalGroup(2, [gmsh_dict[lam_rotor_surf_id]["tag"]])
    model.setPhysicalName(2, pg, gmsh_dict[lam_rotor_surf_id]["label"])

    # Default rotor mesh element size
    mesh_size = machine.stator.Rext / 100.0
    nsurf = 0
    for surf in stator_list:
        nsurf += 1
        gmsh_dict.update({nsurf: {"tag": None, "label": surf.label}})
        if surf.label.find("Lamination_Stator") != -1:
            gmsh_dict[nsurf]["with_holes"] = True
        else:
            gmsh_dict[nsurf]["with_holes"] = False
        if user_mesh_dict is not None:
            mesh_dict = surf.comp_mesh_dict(element_size=mesh_size)
            mesh_dict.update(user_mesh_dict)
        for line in surf.get_lines():
            n_elem = mesh_dict.get(line.label)
            # Gmsh built-in engine does not allow arcs larger than 180deg
            # so arcs are split into two
            if (
                isinstance(line, Arc)
                and abs(line.get_angle() * 180.0 / cmath.pi) == 180.0
            ):
                if line.is_trigo_direction == True:
                    arc1 = Arc2(
                        begin=line.get_begin(),
                        center=line.get_center(),
                        angle=cmath.pi / 2.0,
                        label=line.label,
                    )
                    arc2 = Arc2(
                        begin=arc1.get_end(),
                        center=line.get_center(),
                        angle=cmath.pi / 2.0,
                        label=line.label,
                    )
                else:
                    arc1 = Arc2(
                        begin=line.get_begin(),
                        center=line.get_center(),
                        angle=-cmath.pi / 2.0,
                        label=line.label,
                    )
                    arc2 = Arc2(
                        begin=arc1.get_end(),
                        center=line.get_center(),
                        angle=-cmath.pi / 2.0,
                        label=line.label,
                    )
                if n_elem is not None:
                    _add_line_to_dict(
                        geo=factory,
                        line=arc1,
                        d=gmsh_dict,
                        idx=nsurf,
                        n_elements=n_elem,
                    )
                    _add_line_to_dict(
                        geo=factory,
                        line=arc2,
                        d=gmsh_dict,
                        idx=nsurf,
                        n_elements=n_elem,
                    )
                else:
                    _add_line_to_dict(
                        geo=factory,
                        line=arc1,
                        d=gmsh_dict,
                        idx=nsurf,
                        mesh_size=mesh_size,
                    )
                    _add_line_to_dict(
                        geo=factory,
                        line=arc2,
                        d=gmsh_dict,
                        idx=nsurf,
                        mesh_size=mesh_size,
                    )
            else:
                if n_elem is not None:
                    _add_line_to_dict(
                        geo=factory,
                        line=line,
                        d=gmsh_dict,
                        idx=nsurf,
                        n_elements=n_elem,
                    )
                else:
                    _add_line_to_dict(
                        geo=factory,
                        line=line,
                        d=gmsh_dict,
                        idx=nsurf,
                        mesh_size=mesh_size,
                    )

    for s_id, s_data in gmsh_dict.items():
        lloop = []
        if s_id == 0:
            continue
        for lid, lvalues in s_data.items():
            if type(lvalues) is not dict:
                continue
            lloop.extend([lvalues["tag"]])
        cloop = factory.addCurveLoop(lloop)
        # Winding surfaces are created
        if s_data["label"].find("Lamination_Stator") != -1:
            s_data["tag"] = factory.addPlaneSurface([cloop], tag=-1)
            pg = model.addPhysicalGroup(2, [s_data["tag"]])
            model.setPhysicalName(2, pg, s_data["label"])
        else:
            # Stator lamination is built
            if not is_lam_only_S:
                s_data["tag"] = factory.addPlaneSurface([cloop], tag=-1)
                pg = model.addPhysicalGroup(2, [s_data["tag"]])
                model.setPhysicalName(2, pg, s_data["label"])

    factory.synchronize()
    gmsh.model.mesh.generate(2)

    # Save and close
    gmsh.write(path_save)
    # gmsh.fltk.run()      # Uncomment to launch Gmsh GUI
    gmsh.finalize()

    return gmsh_dict



# -*- coding: utf-8 -*-
from numpy import exp


def rotate(self, angle):
    """Rotation of the Segment of angle

    Parameters
    ----------
    self : Segment
        An Segment Object

    angle : float
        the angle of rotation [rad]

    Returns
    -------
    None

    """
    if not isinstance(angle, float) and not isinstance(angle, int):
        raise AngleRotationSegmentError("The angle must be a float or int ")

    # check if Segment is correct"
    self.check()

    # Modification from the rotation  of the object
    self.begin = self.begin * exp(1j * angle)
    self.end = self.end * exp(1j * angle)


class AngleRotationSegmentError(Exception):
    """ """

    pass



# -*- coding: utf-8 -*-
import numpy as np


def get_axis(self, args=None):
    """Get the axis of variables stored in Solution.

    Parameters
    ----------
    self : Solution
        an Solution object
    field_name : str
        name of the field to return

    Returns
    -------
    field: array
        an array of field values

    """

    axis_dict = dict()
    comp = self.field.components["x"]
    for axis in comp.axes:
        axis_dict[axis.name] = comp.get_along(axis.name)[comp.symbol].size

    axis_dict["component"] = len(self.field.components)

    return axis_dict



# -*- coding: utf-8 -*-

from numpy import angle, arcsin, arctan, array, cos, exp


def comp_radius(self):
    """Compute the radius of the min and max circle that contains the slot

    Parameters
    ----------
    self : HoleM50
        A HoleM50 object

    Returns
    -------
    (Rmin,Rmax): tuple
        Radius of the circle that contains the slot [m]
    """
    Rbo = self.get_Rbo()

    Rmax = Rbo - self.H1

    # magnet pole pitch angle, must be <2*pi/2*p
    alpham = 2 * arcsin(self.W0 / (2 * (Rbo - self.H1)))

    Harc = (Rbo - self.H1) * (1 - cos(alpham / 2))
    gammam = arctan((self.H0 - self.H1 - Harc) / (self.W0 / 2.0 - self.W1 / 2.0))

    x78 = (self.H3 - self.H2) / cos(gammam)  # distance from 7 to 8
    Z9 = Rbo - Harc - self.H1 - 1j * self.W0 / 2.0
    Z8 = Rbo - self.H0 - 1j * self.W1 / 2.0
    Z7 = Rbo - self.H0 - x78 - 1j * self.W1 / 2.0

    # Magnet coordinate with Z8 as center and x as the top edge of the magnet
    Z8b = self.W2
    Z8c = Z8b + self.W4
    Z5 = Z8b - 1j * self.H3
    Z4 = Z8c - 1j * self.H3
    Z6 = Z5 + 1j * self.H2
    Z3 = Z4 + 1j * self.H2

    Zmag = array([Z8b, Z6, Z5, Z4, Z3, Z8c])
    Zmag = Zmag * exp(1j * angle(Z9 - Z8))
    Zmag = Zmag + Z8

    # final complex numbers Zmag=[Z8b Z6 Z5 Z4 Z3 Z8c]
    (Z8b, Z6, Z5, Z4, Z3, Z8c) = Zmag

    Rmin = min(abs(Z5), abs(Z7))

    return (Rmin, Rmax)



# -*- coding: utf-8 -*-

from numpy import arcsin


def comp_angle_opening_magnet(self):
    """Compute the average opening angle of the slot for a single magnet

    Parameters
    ----------
    self : SlotMFlat
        A SlotMFlat object

    Returns
    -------
    alpha: float
        Average opening angle of the slot [rad]

    """

    if self.W0_is_rad:
        return self.W0
    else:  # Convert W0 from m to rad
        Rbo = self.get_Rbo()
        return float(2 * arcsin(self.W0 / (2 * Rbo)))



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/DataKeeper.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/DataKeeper
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ntpath import basename
from os.path import isfile
from ._check import CheckTypeError
import numpy as np
import random
from ._check import InitUnKnowClassError


class DataKeeper(FrozenClass):
    """Abstract class for the multi-simulation"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        name="",
        symbol="",
        unit="",
        keeper=None,
        error_keeper=None,
        result=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "keeper" in list(init_dict.keys()):
                keeper = init_dict["keeper"]
            if "error_keeper" in list(init_dict.keys()):
                error_keeper = init_dict["error_keeper"]
            if "result" in list(init_dict.keys()):
                result = init_dict["result"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.name = name
        self.symbol = symbol
        self.unit = unit
        self.keeper = keeper
        self.error_keeper = error_keeper
        self.result = result

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        DataKeeper_str = ""
        if self.parent is None:
            DataKeeper_str += "parent = None " + linesep
        else:
            DataKeeper_str += "parent = " + str(type(self.parent)) + " object" + linesep
        DataKeeper_str += 'name = "' + str(self.name) + '"' + linesep
        DataKeeper_str += 'symbol = "' + str(self.symbol) + '"' + linesep
        DataKeeper_str += 'unit = "' + str(self.unit) + '"' + linesep
        if self._keeper_str is not None:
            DataKeeper_str += "keeper = " + self._keeper_str + linesep
        elif self._keeper_func is not None:
            DataKeeper_str += "keeper = " + str(self._keeper_func) + linesep
        else:
            DataKeeper_str += "keeper = None" + linesep + linesep
        if self._error_keeper_str is not None:
            DataKeeper_str += "error_keeper = " + self._error_keeper_str + linesep
        elif self._error_keeper_func is not None:
            DataKeeper_str += "error_keeper = " + str(self._error_keeper_func) + linesep
        else:
            DataKeeper_str += "error_keeper = None" + linesep + linesep
        DataKeeper_str += (
            "result = "
            + linesep
            + str(self.result).replace(linesep, linesep + "\t")
            + linesep
        )
        return DataKeeper_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.name != self.name:
            return False
        if other.symbol != self.symbol:
            return False
        if other.unit != self.unit:
            return False
        if other._keeper_str != self._keeper_str:
            return False
        if other._error_keeper_str != self._error_keeper_str:
            return False
        if other.result != self.result:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        DataKeeper_dict = dict()
        DataKeeper_dict["name"] = self.name
        DataKeeper_dict["symbol"] = self.symbol
        DataKeeper_dict["unit"] = self.unit
        if self._keeper_str is not None:
            DataKeeper_dict["keeper"] = self._keeper_str
        else:
            DataKeeper_dict["keeper"] = None
        if self._error_keeper_str is not None:
            DataKeeper_dict["error_keeper"] = self._error_keeper_str
        else:
            DataKeeper_dict["error_keeper"] = None
        DataKeeper_dict["result"] = (
            self.result.copy() if self.result is not None else None
        )
        # The class name is added to the dict for deserialisation purpose
        DataKeeper_dict["__class__"] = "DataKeeper"
        return DataKeeper_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.name = None
        self.symbol = None
        self.unit = None
        self.keeper = None
        self.error_keeper = None
        self.result = None

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""Data name

        :Type: str
        """,
    )

    def _get_symbol(self):
        """getter of symbol"""
        return self._symbol

    def _set_symbol(self, value):
        """setter of symbol"""
        check_var("symbol", value, "str")
        self._symbol = value

    symbol = property(
        fget=_get_symbol,
        fset=_set_symbol,
        doc=u"""Data symbol

        :Type: str
        """,
    )

    def _get_unit(self):
        """getter of unit"""
        return self._unit

    def _set_unit(self, value):
        """setter of unit"""
        check_var("unit", value, "str")
        self._unit = value

    unit = property(
        fget=_get_unit,
        fset=_set_unit,
        doc=u"""Data unit

        :Type: str
        """,
    )

    def _get_keeper(self):
        """getter of keeper"""
        return self._keeper_func

    def _set_keeper(self, value):
        """setter of keeper"""
        if value is None:
            self._keeper_str = None
            self._keeper_func = None
        elif isinstance(value, str) and "lambda" in value:
            self._keeper_str = value
            self._keeper_func = eval(value)
        elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
            self._keeper_str = value
            f = open(value, "r")
            exec(f.read(), globals())
            self._keeper_func = eval(basename(value[:-3]))
        elif callable(value):
            self._keeper_str = None
            self._keeper_func = value
        else:
            raise CheckTypeError(
                "For property keeper Expected function or str (path to python file or lambda), got: "
                + str(type(value))
            )

    keeper = property(
        fget=_get_keeper,
        fset=_set_keeper,
        doc=u"""Function that takes an Output in argument and return a value

        :Type: function
        """,
    )

    def _get_error_keeper(self):
        """getter of error_keeper"""
        return self._error_keeper_func

    def _set_error_keeper(self, value):
        """setter of error_keeper"""
        if value is None:
            self._error_keeper_str = None
            self._error_keeper_func = None
        elif isinstance(value, str) and "lambda" in value:
            self._error_keeper_str = value
            self._error_keeper_func = eval(value)
        elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
            self._error_keeper_str = value
            f = open(value, "r")
            exec(f.read(), globals())
            self._error_keeper_func = eval(basename(value[:-3]))
        elif callable(value):
            self._error_keeper_str = None
            self._error_keeper_func = value
        else:
            raise CheckTypeError(
                "For property error_keeper Expected function or str (path to python file or lambda), got: "
                + str(type(value))
            )

    error_keeper = property(
        fget=_get_error_keeper,
        fset=_set_error_keeper,
        doc=u"""Function that takes a Simulation in argument and returns a value, this attribute enables to handle errors and to put NaN values in the result matrices

        :Type: function
        """,
    )

    def _get_result(self):
        """getter of result"""
        return self._result

    def _set_result(self, value):
        """setter of result"""
        if type(value) is int and value == -1:
            value = list()
        check_var("result", value, "list")
        self._result = value

    result = property(
        fget=_get_result,
        fset=_set_result,
        doc=u"""List containing datakeeper results for each simulation

        :Type: list
        """,
    )



# -*- coding: utf-8 -*-

# File generated according to SMagnet.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from .....GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect
from pyleecan.GUI.Tools.HelpButton import HelpButton

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SMagnet(object):
    def setupUi(self, SMagnet):
        if not SMagnet.objectName():
            SMagnet.setObjectName(u"SMagnet")
        SMagnet.resize(650, 550)
        SMagnet.setMinimumSize(QSize(650, 550))
        self.main_layout = QVBoxLayout(SMagnet)
        self.main_layout.setObjectName(u"main_layout")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.b_help = HelpButton(SMagnet)
        self.b_help.setObjectName(u"b_help")
        self.b_help.setPixmap(QPixmap(u":/images/images/icon/help_16.png"))

        self.horizontalLayout_3.addWidget(self.b_help)

        self.c_type = QComboBox(SMagnet)
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.addItem("")
        self.c_type.setObjectName(u"c_type")
        self.c_type.setMinimumSize(QSize(100, 0))
        self.c_type.setAutoFillBackground(False)

        self.horizontalLayout_3.addWidget(self.c_type)

        self.out_Nmag = QLabel(SMagnet)
        self.out_Nmag.setObjectName(u"out_Nmag")

        self.horizontalLayout_3.addWidget(self.out_Nmag)

        self.horizontalSpacer_3 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_3.addItem(self.horizontalSpacer_3)

        self.main_layout.addLayout(self.horizontalLayout_3)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.w_mat = WMatSelect(SMagnet)
        self.w_mat.setObjectName(u"w_mat")

        self.horizontalLayout.addWidget(self.w_mat)

        self.in_type_magnetization = QLabel(SMagnet)
        self.in_type_magnetization.setObjectName(u"in_type_magnetization")

        self.horizontalLayout.addWidget(self.in_type_magnetization)

        self.c_type_magnetization = QComboBox(SMagnet)
        self.c_type_magnetization.addItem("")
        self.c_type_magnetization.addItem("")
        self.c_type_magnetization.addItem("")
        self.c_type_magnetization.setObjectName(u"c_type_magnetization")

        self.horizontalLayout.addWidget(self.c_type_magnetization)

        self.main_layout.addLayout(self.horizontalLayout)

        self.w_mag = QWidget(SMagnet)
        self.w_mag.setObjectName(u"w_mag")

        self.main_layout.addWidget(self.w_mag)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.b_plot = QPushButton(SMagnet)
        self.b_plot.setObjectName(u"b_plot")

        self.horizontalLayout_2.addWidget(self.b_plot)

        self.b_previous = QPushButton(SMagnet)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout_2.addWidget(self.b_previous)

        self.b_next = QPushButton(SMagnet)
        self.b_next.setObjectName(u"b_next")
        self.b_next.setEnabled(True)

        self.horizontalLayout_2.addWidget(self.b_next)

        self.main_layout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(SMagnet)

        QMetaObject.connectSlotsByName(SMagnet)

    # setupUi

    def retranslateUi(self, SMagnet):
        SMagnet.setWindowTitle(QCoreApplication.translate("SMagnet", u"Form", None))
        self.b_help.setText("")
        self.c_type.setItemText(
            0, QCoreApplication.translate("SMagnet", u"Rectangular", None)
        )
        self.c_type.setItemText(
            1, QCoreApplication.translate("SMagnet", u"Polar", None)
        )
        self.c_type.setItemText(
            2, QCoreApplication.translate("SMagnet", u"Flat bottom, polar top", None)
        )
        self.c_type.setItemText(
            3, QCoreApplication.translate("SMagnet", u"Flat bottom, curved top", None)
        )
        self.c_type.setItemText(
            4, QCoreApplication.translate("SMagnet", u"Polar bottom, curved top", None)
        )

        self.out_Nmag.setText(
            QCoreApplication.translate("SMagnet", u"Number of magnet = 2 * p = ?", None)
        )
        self.in_type_magnetization.setText(
            QCoreApplication.translate("SMagnet", u"type_magnetization:", None)
        )
        self.c_type_magnetization.setItemText(
            0, QCoreApplication.translate("SMagnet", u"Radial", None)
        )
        self.c_type_magnetization.setItemText(
            1, QCoreApplication.translate("SMagnet", u"Parallel", None)
        )
        self.c_type_magnetization.setItemText(
            2, QCoreApplication.translate("SMagnet", u"HallBach", None)
        )

        self.b_plot.setText(QCoreApplication.translate("SMagnet", u"Preview", None))
        self.b_previous.setText(
            QCoreApplication.translate("SMagnet", u"Previous", None)
        )
        self.b_next.setText(QCoreApplication.translate("SMagnet", u"Save", None))

    # retranslateUi



# -*- coding: utf-8 -*-

from numpy import pi
from ....Classes.LamSlot import LamSlot


def comp_masses(self):
    """Compute the Lamination masses (Mlam, Mmag)

    Parameters
    ----------
    self : LamSlotMag
        A LamSlotMag object

    Returns
    -------
    M_dict: dict
        Lamination mass dictionnary (Mtot, Mlam, Mmag) [kg]

    """

    M_dict = LamSlot.comp_masses(self)
    Mmag = 0
    for magnet in self.slot.magnet:
        Mmag += magnet.comp_mass()

    M_dict["Mmag"] = Mmag
    M_dict["Mtot"] += Mmag

    return M_dict



def run(self):
    """Run the simulation and optionnaly its multisimulation

    Returns
    -------
    output: Output
        Contains simulation results
    """
    # In-function import to avoid ImportError
    from ....Classes.XOutput import XOutput
    from ....Classes.Output import Output

    # Multi-simulation
    if self.var_simu is not None:
        # XOutput initialization
        if self.parent is None:
            results = XOutput(simu=self)
        else:
            if not isinstance(self.parent, XOutput):
                raise TypeError(
                    "A multisimulation has been defined, type XOutput expected for simu.parent, got {}".format(
                        type(self.parent).__name__
                    )
                )
            else:
                results = self.parent

        # Compute the reference simulation
        if self.var_simu.ref_simu_index is None:
            self.run_single()

        # Compute the multisimulation
        self.var_simu.run()

    else:
        # Output initialization
        if self.parent is None:
            results = Output(simu=self)
        else:
            results = self.parent

        # Compute the simulation
        self.run_single()

    return results



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/InputFlux.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/InputFlux
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Input import Input

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.InputFlux.gen_input import gen_input
except ImportError as error:
    gen_input = error


from ..Classes.ImportMatrixVal import ImportMatrixVal
from numpy import ndarray
from numpy import array, array_equal
from ._check import InitUnKnowClassError
from .ImportVectorField import ImportVectorField
from .Input import Input
from .ImportMatrix import ImportMatrix


class InputFlux(Input):
    """Input to skip the magnetic module and start with the structural one"""

    VERSION = 1

    # cf Methods.Simulation.InputFlux.gen_input
    if isinstance(gen_input, ImportError):
        gen_input = property(
            fget=lambda x: raise_(
                ImportError("Can't use InputFlux method gen_input: " + str(gen_input))
            )
        )
    else:
        gen_input = gen_input
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        B=None,
        OP=None,
        time=None,
        angle=None,
        Nt_tot=2048,
        Nrev=1,
        Na_tot=2048,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "B" in list(init_dict.keys()):
                B = init_dict["B"]
            if "OP" in list(init_dict.keys()):
                OP = init_dict["OP"]
            if "time" in list(init_dict.keys()):
                time = init_dict["time"]
            if "angle" in list(init_dict.keys()):
                angle = init_dict["angle"]
            if "Nt_tot" in list(init_dict.keys()):
                Nt_tot = init_dict["Nt_tot"]
            if "Nrev" in list(init_dict.keys()):
                Nrev = init_dict["Nrev"]
            if "Na_tot" in list(init_dict.keys()):
                Na_tot = init_dict["Na_tot"]
        # Set the properties (value check and convertion are done in setter)
        self.B = B
        self.OP = OP
        # Call Input init
        super(InputFlux, self).__init__(
            time=time, angle=angle, Nt_tot=Nt_tot, Nrev=Nrev, Na_tot=Na_tot
        )
        # The class is frozen (in Input init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        InputFlux_str = ""
        # Get the properties inherited from Input
        InputFlux_str += super(InputFlux, self).__str__()
        if self.B is not None:
            tmp = self.B.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            InputFlux_str += "B = " + tmp
        else:
            InputFlux_str += "B = None" + linesep + linesep
        if self.OP is not None:
            tmp = self.OP.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            InputFlux_str += "OP = " + tmp
        else:
            InputFlux_str += "OP = None" + linesep + linesep
        return InputFlux_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Input
        if not super(InputFlux, self).__eq__(other):
            return False
        if other.B != self.B:
            return False
        if other.OP != self.OP:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        # Get the properties inherited from Input
        InputFlux_dict = super(InputFlux, self).as_dict()
        if self.B is None:
            InputFlux_dict["B"] = None
        else:
            InputFlux_dict["B"] = self.B.as_dict()
        if self.OP is None:
            InputFlux_dict["OP"] = None
        else:
            InputFlux_dict["OP"] = self.OP.as_dict()
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        InputFlux_dict["__class__"] = "InputFlux"
        return InputFlux_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.B is not None:
            self.B._set_None()
        if self.OP is not None:
            self.OP._set_None()
        # Set to None the properties inherited from Input
        super(InputFlux, self)._set_None()

    def _get_B(self):
        """getter of B"""
        return self._B

    def _set_B(self, value):
        """setter of B"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "B")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = ImportVectorField()
        check_var("B", value, "ImportVectorField")
        self._B = value

        if self._B is not None:
            self._B.parent = self

    B = property(
        fget=_get_B,
        fset=_set_B,
        doc=u"""Airgap flux density

        :Type: ImportVectorField
        """,
    )

    def _get_OP(self):
        """getter of OP"""
        return self._OP

    def _set_OP(self, value):
        """setter of OP"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "OP")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Input()
        check_var("OP", value, "Input")
        self._OP = value

        if self._OP is not None:
            self._OP.parent = self

    OP = property(
        fget=_get_OP,
        fset=_set_OP,
        doc=u"""InputCurrent to define Operating Point (not mandatory)

        :Type: Input
        """,
    )



from numpy import angle, arcsin, exp, tan
from ....Methods import ParentMissingError


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : MagnetType11
        A MagnetType11 object

    Returns
    -------
    point_list: list
        A list of 6 complex coordinates

    """
    if self.parent is not None:
        (Z1, Z2) = self.parent.get_point_bottom()
        H0 = self.parent.H0
        W0 = self.parent.W0
    else:
        raise ParentMissingError(
            "Error: The magnet object is not inside a " + "slot object"
        )

    # comp point coordinate (in complex)
    if W0 > self.Wmag:  # The magnet is smaller than the slot => center the mag
        Z1 = Z1 * exp(1j * (W0 - self.Wmag) / 2)
        Z2 = Z2 * exp(-1j * (W0 - self.Wmag) / 2)

    Rbot = abs(Z1)
    if self.is_outwards():
        Rtop = Rbot - self.Hmag
        Rslot = Rbot - H0
        Zref = Rbot - self.Hmag / 2
    else:
        Rtop = Rbot + self.Hmag
        Rslot = Rbot + H0
        Zref = Rbot + self.Hmag / 2

    Z3 = Rtop * exp(1j * angle(Z1))
    Zs3 = Rslot * exp(1j * angle(Z1))
    Z4 = Rtop * exp(1j * angle(Z2))
    Zs4 = Rslot * exp(1j * angle(Z2))

    return [Z1, Z2, Z3, Zs3, Zs4, Z4, Zref]



# -*- coding: utf-8 -*-


def comp_surface_active(self):
    """Compute the active surface of the conductor

    Parameters
    ----------
    self : CondType22
        A CondType22 object

    Returns
    -------
    Sact: float
        Surface without insulation [m**2]

    """

    return self.Sbar



from numpy import arcsin, cos, exp, pi, sin, tan, sqrt


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotW14
        A SlotW14 object

    Returns
    -------
    point_list: list
        A list of 9 Points and W2

    """
    Rbo = self.get_Rbo()
    hssp = pi / self.Zs

    # alpha is the angle to rotate P0 so ||P1,P9|| = W0
    alpha = arcsin(self.W0 / (2 * Rbo))
    Harc = float(Rbo * (1 - cos(alpha)))

    Z9 = Rbo * exp(-1j * alpha)
    if self.is_outwards():
        R1 = Rbo - Harc + self.H0 + self.H1
        Z8 = Z9 + self.H0
    else:
        R1 = Rbo - Harc - self.H0 - self.H1
        Z8 = Z9 - self.H0
    # In the slot ref: Z7=x7+j*y7 with x7 = R1
    # In the tooth ref: Zt7 = xt7 + j*yt7 with yt7 = W3/2
    # Zt7 = Z7 * exp(1j*hsp) = (x7+jy7)*(cos(hsp)+1j*sin(hsp))
    # yt7 = W3/2 = x7*sin(hsp)+y7*cos(hsp)
    Z7 = R1 + 1j * (self.W3 / 2 - R1 * sin(hssp)) / cos(hssp)

    # Z7t is Z7 in tooth ref
    Z7t = Z7 * exp(1j * hssp)
    if self.is_outwards():
        Z6t = Z7t + self.H3
    else:
        Z6t = Z7t - self.H3
    # In the slot ref: Z5=x5+j*y5 with y5 = 0
    # In the tooth ref: Zt5 = xt5 + j*yt5 with xt5 = xt6
    # Zt5 = Z5 * exp(1j*hsp) = x5*cos(hsp)+1j*x5*sin(hsp)
    # x5 = real(Z6t)/cos(hsp)

    Z5 = Z6t.real / cos(hssp)
    Z6 = Z6t * exp(-1j * hssp)

    # Symmetry
    Z4 = Z6.conjugate()
    Z3 = Z7.conjugate()
    Z2 = Z8.conjugate()
    Z1 = Z9.conjugate()

    return [Z9, Z8, Z7, Z6, Z5, Z4, Z3, Z2, Z1]



# -*- coding: utf-8 -*-

from ....Classes.Arc1 import Arc1
from ....Classes.Arc3 import Arc3
from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve (Line) needed to plot the object.
    The ending point of a curve is the starting point of the next curve
    in the list

    Parameters
    ----------
    self : SlotW11
        A SlotW11 object

    Returns
    -------
    curve_list: list
        A list of 7 Segment and 2 Arc1

    """

    [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, Z9, Z10, rot_sign] = self._comp_point_coordinate()

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z4))
    if self.R1 * 2 < self.W2:
        curve_list.append(
            Arc1(Z4, Z5, rot_sign * self.R1, is_trigo_direction=self.is_outwards())
        )
        curve_list.append(Segment(Z5, Z6))
        curve_list.append(
            Arc1(Z6, Z7, rot_sign * self.R1, is_trigo_direction=self.is_outwards())
        )
    else:
        curve_list.append(Arc3(Z4, Z7, self.is_outwards()))
    curve_list.append(Segment(Z7, Z8))
    curve_list.append(Segment(Z8, Z9))
    curve_list.append(Segment(Z9, Z10))

    return curve_list



# -*- coding: utf-8 -*-


def get_lam_list(self, is_int_to_ext=True):
    """Returns the ordered list of lamination of the machine

    Parameters
    ----------
    self : MachineUD
        MachineUD object
    is_int_to_ext : bool
        true to order the list from the inner lamination to the extrenal one

    Returns
    -------
    lam_list : list
        Ordered lamination list
    """

    lam_list = self.lam_list

    # Sort by Rint by assuming the lamination are not colliding
    return sorted(lam_list, key=lambda x: x.Rint, reverse=not is_int_to_ext)



# -*- coding: utf-8 -*-
from matplotlib.pyplot import axis, subplots
from ....Functions.init_fig import init_fig


def plot(
    self, fig=None, sym=1, alpha=0, delta=0, is_edge_only=False, comp_machine=None
):
    """Plot the Machine in a matplotlib fig

    Parameters
    ----------
    self : MachineUD
        A MachineUD object
    fig :
        if None, open a new fig and plot, else add to the gcf (Default value = None)
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_edge_only: bool
        To plot transparent Patches
    comp_machine : Machine
        A machine to plot in transparency on top of the self machine

    Returns
    -------
    None

    """
    # Display
    # fig, axes = subplots()
    (fig, axes, patch_leg, label_leg) = init_fig(fig)

    plot_list = list()
    # Get the patches to display from corresponding plot
    # The order in the list matters (largest to smallest)
    if self.frame is not None:
        plot_list.append(self.frame.plot)
        Wfra = self.frame.comp_height_eq()
    else:
        Wfra = 0

    Rext = 0
    for lam in self.lam_list:
        plot_list.append(lam.plot)
        Rext = max(Rext, lam.Rext)

    Lim = (Rext + Wfra) * 1.5  # Axes limit for plot

    # Plot
    plot_args = {
        "sym": sym,
        "alpha": alpha,
        "delta": delta,
        "is_edge_only": is_edge_only,
    }

    for plot_fct in plot_list:
        _plot(plot_fct, fig, plot_args)

    if comp_machine is not None:
        raise ValueError("Comp_machine is not available for MachineUD")
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    if self.name in [None, ""]:
        axes.set_title("Machine")
    else:
        axes.set_title(self.name)

    # Axis Setup
    axis("equal")

    # The Lamination is centered in the figure
    axes.set_xlim(-Lim, Lim)
    axes.set_ylim(-Lim, Lim)
    fig.show()


def _plot(plt_fcn, fig, plot_args):
    if plt_fcn is not None:
        try:
            plt_fcn(fig, **plot_args)
        except Exception:
            pass



from numpy import pi
from SciDataTool import Data1D, DataLinspace
from ....Methods.Simulation.Input import InputError


def comp_axes(self, machine, N0=None):
    """Compute simulation axes, i.e. space DataObject including (anti)-periodicity
    and time DataObject including (anti)-periodicity and accounting for rotating speed
    and number of revolutions

    Parameters
    ----------
    self : Input
        an Input object
    machine : Machine
        a Machine object
    N0 : float
        rotating speed [rpm]


    Returns
    -------
    Time : DataLinspace
        Time axis including (anti)-periodicity and accounting for rotating speed and number of revolutions
    Angle : DataLinspace
        Angle axis including (anti)-periodicity

    """

    # Time axis
    if self.time is None:
        if N0 is None:
            raise InputError("ERROR: time and N0 can't be both None")

        Time = DataLinspace(
            name="time",
            unit="s",
            initial=0,
            final=60 / N0 * self.Nrev,
            number=self.Nt_tot,
            include_endpoint=False,
        )
    else:
        # Load and check time
        time = self.time.get_data()
        self.Nt_tot = len(time)
        Time = Data1D(name="time", unit="s", values=time)

    # Angle axis
    if self.angle is None:
        # Create angle vector as a linspace
        Angle = DataLinspace(
            name="angle",
            unit="rad",
            initial=0,
            final=2 * pi,
            number=self.Na_tot,
            include_endpoint=False,
        )
    else:
        # Load angle data
        angle = self.angle.get_data()
        self.Na_tot = len(angle)
        Angle = Data1D(name="angle", unit="rad", values=angle)

    return Time, Angle



# -*- coding: utf-8 -*-

from numpy import angle, exp

from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine
from ....Functions.Geometry.comp_flower_arc import comp_flower_arc
from ....Methods import ParentMissingError


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Segment, Arc1) needed to plot the Magnet.
    The list represents a closed surface.
    The ending point of a curve is always the starting point of the next
    curve in the list

    Parameters
    ----------
    self : MagnetType14
        A MagnetType14 object
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_simplified: bool
        True to avoid line superposition

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the lamination

    """

    # defining label for type_magnetization
    if self.type_magnetization == 0:
        t_p = "Radial"
    else:
        t_p = "Parallel"

    if self.parent is not None:
        (Z1, Z2) = self.parent.get_point_bottom()
        H0 = self.parent.H0
        W0 = self.parent.W0
    else:
        raise ParentMissingError(
            "Error: The magnet object is not inside a " + "slot object"
        )

    # comp point coordinate (in complex)
    if W0 > self.Wmag:  # The magnet is smaller than the slot => center the mag
        Z1 = Z1 * exp(1j * (W0 - self.Wmag) / 2)
        Z2 = Z2 * exp(-1j * (W0 - self.Wmag) / 2)

    if self.is_outwards():
        (alpha_lim, Z4, Z3) = comp_flower_arc(
            abs(angle(Z1) - angle(Z2)), self.Rtop, abs(Z1) - self.Hmag
        )
        Zs3 = (abs(Z1) - H0) * exp(1j * angle(Z1))
        Zs4 = (abs(Z2) - H0) * exp(1j * angle(Z2))
        Zref = abs(Z1) - self.Hmag / 2
    else:
        (alpha_lim, Z4, Z3) = comp_flower_arc(
            abs(angle(Z1) - angle(Z2)), self.Rtop, abs(Z1) + self.Hmag
        )
        Zs3 = (abs(Z1) + H0) * exp(1j * angle(Z1))
        Zs4 = (abs(Z2) + H0) * exp(1j * angle(Z2))
        Zref = abs(Z1) + self.Hmag / 2

    # Creation of curve
    curve_list = list()
    if is_simplified and W0 > self.Wmag:
        curve_list.append(Segment(Z1, Z3))
    elif is_simplified and H0 < self.Hmag:
        curve_list.append(Segment(Zs3, Z3))
    elif not is_simplified:
        curve_list.append(Segment(Z1, Z3))

    curve_list.append(Arc1(Z3, Z4, self.Rtop))

    if is_simplified and W0 > self.Wmag:
        curve_list.append(Segment(Z4, Z2))
    elif is_simplified and H0 < self.Hmag:
        curve_list.append(Segment(Z4, Zs4))
    elif not is_simplified:
        curve_list.append(Segment(Z4, Z2))

    if not is_simplified:
        curve_list.append(Arc1(Z2, Z1, -abs(Z2), is_trigo_direction=False))

    surf_list = list()
    surf_list.append(
        SurfLine(
            line_list=curve_list,
            label="MagnetRotor" + t_p + "_N_R0_T0_S0",
            point_ref=Zref,
        )
    )

    # Apply transformation
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Import/ImportMatlab.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Import/ImportMatlab
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .ImportMatrix import ImportMatrix

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Import.ImportMatlab.get_data import get_data
except ImportError as error:
    get_data = error


from ._check import InitUnKnowClassError


class ImportMatlab(ImportMatrix):
    """Import the data from a mat file"""

    VERSION = 1

    # cf Methods.Import.ImportMatlab.get_data
    if isinstance(get_data, ImportError):
        get_data = property(
            fget=lambda x: raise_(
                ImportError("Can't use ImportMatlab method get_data: " + str(get_data))
            )
        )
    else:
        get_data = get_data
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        file_path="",
        var_name="",
        is_transpose=False,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "file_path" in list(init_dict.keys()):
                file_path = init_dict["file_path"]
            if "var_name" in list(init_dict.keys()):
                var_name = init_dict["var_name"]
            if "is_transpose" in list(init_dict.keys()):
                is_transpose = init_dict["is_transpose"]
        # Set the properties (value check and convertion are done in setter)
        self.file_path = file_path
        self.var_name = var_name
        # Call ImportMatrix init
        super(ImportMatlab, self).__init__(is_transpose=is_transpose)
        # The class is frozen (in ImportMatrix init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        ImportMatlab_str = ""
        # Get the properties inherited from ImportMatrix
        ImportMatlab_str += super(ImportMatlab, self).__str__()
        ImportMatlab_str += 'file_path = "' + str(self.file_path) + '"' + linesep
        ImportMatlab_str += 'var_name = "' + str(self.var_name) + '"' + linesep
        return ImportMatlab_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from ImportMatrix
        if not super(ImportMatlab, self).__eq__(other):
            return False
        if other.file_path != self.file_path:
            return False
        if other.var_name != self.var_name:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        # Get the properties inherited from ImportMatrix
        ImportMatlab_dict = super(ImportMatlab, self).as_dict()
        ImportMatlab_dict["file_path"] = self.file_path
        ImportMatlab_dict["var_name"] = self.var_name
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        ImportMatlab_dict["__class__"] = "ImportMatlab"
        return ImportMatlab_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.file_path = None
        self.var_name = None
        # Set to None the properties inherited from ImportMatrix
        super(ImportMatlab, self)._set_None()

    def _get_file_path(self):
        """getter of file_path"""
        return self._file_path

    def _set_file_path(self, value):
        """setter of file_path"""
        check_var("file_path", value, "str")
        self._file_path = value

    file_path = property(
        fget=_get_file_path,
        fset=_set_file_path,
        doc=u"""Path of the file to load

        :Type: str
        """,
    )

    def _get_var_name(self):
        """getter of var_name"""
        return self._var_name

    def _set_var_name(self, value):
        """setter of var_name"""
        check_var("var_name", value, "str")
        self._var_name = value

    var_name = property(
        fget=_get_var_name,
        fset=_set_var_name,
        doc=u"""Name of the variable to load

        :Type: str
        """,
    )



# -*- coding: utf-8 -*-

from .....Functions.Plot.plot_A_fft_space import (
    plot_A_fft_space as plot_A_fft_space_fct,
)
from .....Functions.init_fig import init_fig
from SciDataTool import VectorField


def plot_A_fft_space(
    self,
    Data_str,
    t=None,
    t_index=0,
    is_spaceorder=False,
    r_max=100,
    fund_harm=None,
    is_norm=False,
    unit="SI",
    data_list=[],
    component_list=None,
    legend_list=[],
    color_list=[],
    save_path=None,
    mag_max=None,
    is_auto_ticks=True,
    fig=None,
):
    """Plots a field as a function of space (angle)

    Parameters
    ----------
    self : Output
        an Output object
    Data_str : str
        name of the Data object to plot (e.g. "mag.Br")
    t : float
        time value at which to slice
    t_index : int
        time index at which to slice
    is_spaceorder : bool
        boolean indicating if we want to use the spatial order for the fft axis
    r_max : int
        maximum value of the wavenumber for the fft axis
    fund_harm : float
        frequency of the fundamental harmonic
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    data_list : list
        list of Data objects to compare
    component_list : list
        list of component names to plot in separate figures
    legend_list : list
        list of legends to use for each Data object (including reference one) instead of data.name
    color_list : list
        list of colors to use for each Data object
    save_path : str
        path and name of the png file to save
    mag_max : float
        maximum alue for the y-axis of the fft
    is_auto_ticks : bool
        in fft, adjust ticks to wavenumbers (deactivate if too close)
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    """

    # Get Data object names
    phys = getattr(self, Data_str.split(".")[0])
    data = getattr(phys, Data_str.split(".")[1])

    # Set figure
    is_show_fig = True if fig is None else False
    if fig is None:
        (fig, axes, patch_leg, label_leg) = init_fig(None, shape="rectangle")

    # Call the plot function
    # Call the plot function
    if isinstance(data, VectorField):
        if component_list is None:  # default: extract all components
            component_list = data.components.keys()
        for comp in component_list:
            plot_A_fft_space_fct(
                data.components[comp],
                t=t,
                t_index=t_index,
                is_spaceorder=is_spaceorder,
                r_max=r_max,
                fund_harm=fund_harm,
                unit=unit,
                data_list=[dat.components[comp] for dat in data_list],
                legend_list=legend_list,
                color_list=color_list,
                save_path=save_path,
                mag_max=mag_max,
                is_auto_ticks=is_auto_ticks,
                fig=fig,
            )

    else:
        plot_A_fft_space_fct(
            data,
            t=t,
            t_index=t_index,
            is_spaceorder=is_spaceorder,
            r_max=r_max,
            fund_harm=fund_harm,
            unit=unit,
            data_list=data_list,
            legend_list=legend_list,
            color_list=color_list,
            save_path=save_path,
            mag_max=mag_max,
            is_auto_ticks=is_auto_ticks,
            fig=fig,
        )

    if is_show_fig:
        fig.show()



# -*- coding: utf-8 -*-


def comp_angle_opening(self):
    """Compute the average opening angle of the Slot

    Parameters
    ----------
    self : SlotMFlat2
        A SlotMFlat2 object

    Returns
    -------
    alpha: float
        Average opening angle of the slot [rad]

    """

    alpha0 = self.comp_angle_opening_magnet()
    alpha3 = self.W3

    Nmag = len(self.magnet)
    if Nmag > 0:
        return alpha0 * Nmag + alpha3 * (Nmag - 1)
    else:
        return 0



# -*- coding: utf-8 -*-


def get_lam_list(self, is_int_to_ext=True):
    """Returns the ordered list of lamination of the machine

    Parameters
    ----------
    self : MachineWRSM
        MachineWRSM object
    is_int_to_ext : bool
        true to order the list from the inner lamination to the extrenal one

    Returns
    -------
    lam_list : list
        Ordered lamination list
    """
    if self.rotor.is_internal:
        In = self.rotor
        Ext = self.stator
    else:
        In = self.stator
        Ext = self.rotor

    if is_int_to_ext:
        return [In, Ext]
    else:
        return [Ext, In]



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Mesh/Interpolation/FPGNTri.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Mesh/FPGNTri
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .GaussPoint import GaussPoint

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Mesh.FPGNTri.get_gauss_points import get_gauss_points
except ImportError as error:
    get_gauss_points = error


from ._check import InitUnKnowClassError


class FPGNTri(GaussPoint):
    """Store gauss point for triangle elements"""

    VERSION = 1

    # cf Methods.Mesh.FPGNTri.get_gauss_points
    if isinstance(get_gauss_points, ImportError):
        get_gauss_points = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use FPGNTri method get_gauss_points: "
                    + str(get_gauss_points)
                )
            )
        )
    else:
        get_gauss_points = get_gauss_points
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, nb_gauss_point=3, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "nb_gauss_point" in list(init_dict.keys()):
                nb_gauss_point = init_dict["nb_gauss_point"]
        # Set the properties (value check and convertion are done in setter)
        self.nb_gauss_point = nb_gauss_point
        # Call GaussPoint init
        super(FPGNTri, self).__init__()
        # The class is frozen (in GaussPoint init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        FPGNTri_str = ""
        # Get the properties inherited from GaussPoint
        FPGNTri_str += super(FPGNTri, self).__str__()
        FPGNTri_str += "nb_gauss_point = " + str(self.nb_gauss_point) + linesep
        return FPGNTri_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from GaussPoint
        if not super(FPGNTri, self).__eq__(other):
            return False
        if other.nb_gauss_point != self.nb_gauss_point:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        # Get the properties inherited from GaussPoint
        FPGNTri_dict = super(FPGNTri, self).as_dict()
        FPGNTri_dict["nb_gauss_point"] = self.nb_gauss_point
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        FPGNTri_dict["__class__"] = "FPGNTri"
        return FPGNTri_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.nb_gauss_point = None
        # Set to None the properties inherited from GaussPoint
        super(FPGNTri, self)._set_None()

    def _get_nb_gauss_point(self):
        """getter of nb_gauss_point"""
        return self._nb_gauss_point

    def _set_nb_gauss_point(self, value):
        """setter of nb_gauss_point"""
        check_var("nb_gauss_point", value, "int")
        self._nb_gauss_point = value

    nb_gauss_point = property(
        fget=_get_nb_gauss_point,
        fset=_set_nb_gauss_point,
        doc=u"""Nb of gauss point to be used

        :Type: int
        """,
    )



def run_single_simu(
    xoutput,
    datakeeper_list,
    simulation,
    index,
    stop_if_error,
    ref_simu_index,
    is_keep_all_output,
):
    """
    Execute a simulation and run datakeepers

    Parameters:
    -----------
    xoutput: XOutput
        Contains results
    datakeeper_list
        List of datakeeper to extract results
    simulation: Simulation
        Simulation to run
    index: int
        Index of the simulation
    stop_if_error: bool
        Raises an error if the simulation fails
    ref_simu_index: tuple
        Index of the reference simulation
    is_keep_all_output: bool
        store simulation output
    """
    if stop_if_error:
        is_error = False
        result = simulation.run()
    else:
        # Run simulation
        try:
            result = simulation.run()
            is_error = False
        except Exception as err:
            print(err)
            is_error = True

    # Extract results
    if is_keep_all_output:
        xoutput.output_list.append(result)

    # Datakeepers
    if is_error:
        # Execute error_keeper if error
        for datakeeper in datakeeper_list:
            if datakeeper.error_keeper is None:
                xoutput.xoutput_dict[datakeeper.symbol].result[index] = None
            else:
                xoutput.xoutput_dict[datakeeper.symbol].result[
                    index
                ] = datakeeper.error_keeper(simulation)
    else:
        if index == ref_simu_index:
            for attr in dir(result):
                if (
                    # Not method
                    not callable(getattr(result, attr))
                    # Not private properties
                    and not attr.startswith("_")
                    # Not following properties
                    and attr not in ["VERSION", "logger_name", "parent"]
                ):
                    setattr(xoutput, attr, getattr(result, attr))

        # Execute keepers
        for datakeeper in datakeeper_list:
            xoutput.xoutput_dict[datakeeper.symbol].result[index] = datakeeper.keeper(
                result
            )



# -*- coding: utf-8 -*-

from numpy import angle
from scipy.optimize import fsolve

from ....Classes.Arc1 import Arc1
from ....Classes.Arc3 import Arc3
from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine


def build_geometry_wind(self, Nrad, Ntan, is_simplified=False, alpha=0, delta=0):
    """Split the slot winding area in several zone

    Parameters
    ----------
    self : SlotW28
        A SlotW28 object
    Nrad : int
        Number of radial layer
    Ntan : int
        Number of tangentiel layer
    is_simplified : bool
        boolean to specify if coincident lines are considered as one or different lines (Default value = False)
    alpha : float
        Angle for rotation (Default value = 0) [rad]
    delta : Complex
        complex for translation (Default value = 0)

    Returns
    -------
    surf_list: list
        List of surface delimiting the winding zone

    """
    if self.get_is_stator():  # check if the slot is on the stator
        st = "S"
    else:
        st = "R"

    [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, rot_sign] = self._comp_point_coordinate()
    if self.is_outwards():
        Ztan2 = Z5 + Z5.imag * (1 - 1j)
    else:
        Ztan2 = Z5 + Z5.imag * (-1 - 1j)

    Rarc = abs(Z5.imag)  # Radius of the top arc
    Ztan1 = (Z2 + Z7) / 2.0
    Zmid = (Ztan1 + Ztan2) / 2.0

    # Zrad1 between Z5 and Z6
    x = fsolve(lambda x: angle((Z6 - (Zmid + 1j * x)) / (Z6 - Z5)), -self.R1)
    Zrad1 = Zmid + 1j * x[0]
    Zrad2 = Zrad1.conjugate()

    Zmid = (Ztan1 + Ztan2) / 2.0

    # We can split in rad only if Zrad1 is between Z6 and Z5
    is_rad_splittable = self.H3 > 0 and (
        (Z6.real < Zrad1.real and Zrad1.real < Z5.real)
        or (Z5.real < Zrad1.real and Zrad1.real < Z6.real)
    )

    # Creation of curve
    surf_list = list()
    if Nrad == 1 and Ntan == 2:
        if is_simplified:  # No doubling Line allowed
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z7, Ztan1))
            curve_list.append(Segment(Ztan1, Ztan2))
            point_ref = (Z7 + Ztan1 + Ztan2 + Z5 + Z6) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (0,1)
            curve_list = list()
            curve_list.append(Segment(Ztan1, Z2))
            point_ref = (Ztan1 + Z2 + Z3 + Z4 + Ztan2) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
        else:
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Ztan1, Z2))
            curve_list.append(Arc1(Z2, Z3, rot_sign * self.R1, self.is_outwards()))
            curve_list.append(Segment(Z3, Z4))
            curve_list.append(Arc1(Z4, Ztan2, rot_sign * Rarc, self.is_outwards()))
            curve_list.append(Segment(Ztan2, Ztan1))
            point_ref = (Ztan1 + Z2 + Z3 + Z4 + Ztan2) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (0,1)
            curve_list = list()
            curve_list.append(Segment(Z7, Ztan1))
            curve_list.append(Segment(Ztan1, Ztan2))
            curve_list.append(Arc1(Ztan2, Z5, rot_sign * Rarc, self.is_outwards()))
            curve_list.append(Segment(Z5, Z6))
            curve_list.append(Arc1(Z6, Z7, rot_sign * self.R1, self.is_outwards()))
            point_ref = (Z7 + Ztan1 + Ztan2 + Z5 + Z6) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)

    elif Nrad == 2 and Ntan == 1 and is_rad_splittable:
        if is_simplified:  # No doubling Line allowed
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z7, Z2))
            curve_list.append(Segment(Zrad2, Zrad1))
            point_ref = (Z7 + Z2 + Z3 + Zrad2 + Zrad1 + Z6) / 6
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (1,0)
            curve_list = list()
            point_ref = (Zrad1 + Zrad2 + Z4 + Z5) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
        else:
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z7, Z2))
            curve_list.append(Arc1(Z2, Z3, rot_sign * self.R1, self.is_outwards()))
            curve_list.append(Segment(Z3, Zrad2))
            curve_list.append(Segment(Zrad2, Zrad1))
            curve_list.append(Segment(Zrad1, Z6))
            curve_list.append(Arc1(Z6, Z7, rot_sign * self.R1, self.is_outwards()))
            point_ref = (Z7 + Z2 + Z3 + Zrad2 + Zrad1 + Z6) / 6
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (1,0)
            curve_list = list()
            curve_list.append(Segment(Zrad1, Zrad2))
            curve_list.append(Segment(Zrad2, Z4))
            curve_list.append(Arc3(Z4, Z5, self.is_outwards()))
            curve_list.append(Segment(Z5, Zrad1))
            point_ref = (Zrad1 + Zrad2 + Z4 + Z5) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)

    elif Nrad == 2 and Ntan == 2 and is_rad_splittable:
        if is_simplified:  # No doubling Line allowed
            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z7, Ztan1))
            curve_list.append(Segment(Ztan1, Zmid))
            curve_list.append(Segment(Zmid, Zrad1))
            point_ref = (Z7 + Ztan1 + Zmid + Zrad1 + Z6) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (1,0)
            curve_list = list()
            curve_list.append(Segment(Zmid, Ztan2))
            point_ref = (Zrad1 + Zmid + Ztan2 + Z5) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part 3 (0,1)
            curve_list = list()
            curve_list.append(Segment(Ztan1, Z2))
            curve_list.append(Segment(Zrad2, Zmid))
            point_ref = (Ztan1 + Z2 + Z3 + Zrad2 + Zmid) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part4 (1,1)
            curve_list = list()
            point_ref = (Zmid + Zrad2 + Z4 + Ztan2) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
        else:

            # Part 1 (0,0)
            curve_list = list()
            curve_list.append(Segment(Z7, Ztan1))
            curve_list.append(Segment(Ztan1, Zmid))
            curve_list.append(Segment(Zmid, Zrad1))
            curve_list.append(Segment(Zrad1, Z6))
            curve_list.append(Arc1(Z6, Z7, rot_sign * self.R1, self.is_outwards()))
            point_ref = (Z7 + Ztan1 + Zmid + Zrad1 + Z6) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part2 (1,0)
            curve_list = list()
            curve_list.append(Segment(Zrad1, Zmid))
            curve_list.append(Segment(Zmid, Ztan2))
            curve_list.append(Arc1(Ztan2, Z5, rot_sign * Rarc, self.is_outwards()))
            curve_list.append(Segment(Z5, Zrad1))
            point_ref = (Zrad1 + Zmid + Ztan2 + Z5) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part 3 (0,1)
            curve_list = list()
            curve_list.append(Segment(Ztan1, Z2))
            curve_list.append(Arc1(Z2, Z3, rot_sign * self.R1, self.is_outwards()))
            curve_list.append(Segment(Z3, Zrad2))
            curve_list.append(Segment(Zrad2, Zmid))
            curve_list.append(Segment(Zmid, Ztan1))
            point_ref = (Ztan1 + Z2 + Z3 + Zrad2 + Zmid) / 5
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R0_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part4 (1,1)
            curve_list = list()
            curve_list.append(Segment(Zmid, Zrad2))
            curve_list.append(Segment(Zrad2, Z4))
            curve_list.append(Arc1(Z4, Ztan2, rot_sign * Rarc, self.is_outwards()))
            curve_list.append(Segment(Ztan2, Zmid))
            point_ref = (Zmid + Zrad2 + Z4 + Ztan2) / 4
            surface = SurfLine(
                line_list=curve_list,
                label="Wind" + st + "_R1_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
    else:  # Default : only one zone
        # Only one zone for type 2_6 for now
        curve_list = self.build_geometry()
        # Remove the isthmus part
        curve_list = curve_list[1:-1]
        curve_list = [Segment(curve_list[-1].end, curve_list[0].begin)] + curve_list
        surface = SurfLine(
            line_list=curve_list, label="Wind" + st + "_R0_T0_S0", point_ref=Zmid
        )
        surf_list.append(surface)
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



from SciDataTool.Functions import AxisError


def get_axes(self, output, is_remove_apera=False, is_remove_apert=False):
    """
    Parameters
    ----------
    self : Force
        a Force object
    output: Output
        an Output object
    is_remove_apera: bool
        True to remove anti-periodicity from angular axis if any
    is_remove_apert: bool
        True to remove anti-periodicity from time axis if any

    Returns
    -------
    Time_comp : Data
        a Data object such as Data1D or DataLinspace
    Angle_comp : Data
        a Data object such as Data1D or DataLinspace

    """

    # Periodicity optimization
    (
        per_a,
        is_antiper_a,
        per_t,
        is_antiper_t,
    ) = self.parent.parent.get_machine_periodicity()

    # Getting the computation axes (with or without periodicity)
    if self.is_periodicity_a:
        try:
            # Reduce angle axis to the machine periodicity
            per_a = per_a * 2 if is_antiper_a else per_a
            Angle_comp = output.mag.angle.get_axis_periodic(
                per_a, is_antiper_a and not is_remove_apera
            )

        except AxisError:
            Angle_comp = output.mag.angle
            self.is_periodicity_a = False
            Na_tot = Angle_comp.get_length(is_oneperiod=False)
            self.get_logger().warning(
                "WARNING: In Force model, Na_tot="
                + str(Na_tot)
                + " is not divisible by the machine angular periodicity ("
                + str(per_a)
                + "). Angular periodicity removed"
            )
    else:
        # Return full axis
        Angle_comp = output.mag.angle

    if self.is_periodicity_t:
        try:
            # Reduce time axis to the machine periodicity
            per_t = per_t * 2 if is_antiper_t else per_t
            Time_comp = output.mag.time.get_axis_periodic(
                per_t, is_antiper_t and not is_remove_apert
            )

        except AxisError:
            # Disable periodicity
            Time_comp = output.mag.time
            self.is_periodicity_t = False
            Nt_tot = Time_comp.get_length(is_oneperiod=False)
            self.get_logger().warning(
                "WARNING: In Force model, Nt_tot="
                + str(Nt_tot)
                + " is not divisible by the machine time periodicity ("
                + str(per_t)
                + "). Time periodicity removed"
            )
    else:
        # Return full axis
        Time_comp = output.mag.time

        return Angle_comp, Time_comp



from ....Classes.DataKeeper import DataKeeper
from ....Methods.Simulation.VarLoad.get_elec_datakeeper import (
    get_elec_datakeeper as get_dk,
)


def get_elec_datakeeper(self):
    """
    Generate DataKeepers to store by default results from electric module

    Parameters
    ----------
    self: VarLoad object

    Returns
    -------
    dk_list: list
        list of DataKeeper
    """
    dk_list = get_dk(self)
    if self.type_OP_matrix == 0:  # I0 and Phi0
        # Save I0
        dk_list.append(
            DataKeeper(
                name="I0",
                symbol="I0",
                unit="A",
                keeper="lambda output: np.abs(output.elec.Id_ref + 1j * output.elec.Iq_ref)",
            )
        )
        # Save Phi0
        dk_list.append(
            DataKeeper(
                name="Phi0",
                symbol="Phi0",
                unit="",
                keeper="lambda output: np.angle(output.elec.Id_ref + 1j * output.elec.Iq_ref) % (2 * np.pi)",
            )
        )
    # Keep torque
    dk_list.append(
        DataKeeper(
            name="Reference Average Torque",
            symbol="Tem_av_ref",
            unit="N.m",
            keeper="lambda output: output.elec.Tem_av_ref",
        )
    )

    # TODO Save power
    # if self.is_power:
    #     datakeeper_list.append(
    #         DataKeeper(
    #             name="Power",
    #             symbol="Tem",
    #             unit="N.m",
    #             keeper="lambda output: output.simu.mag.power",
    #         ),
    #     )

    return dk_list



# -*- coding: utf-8 -*-

pixmap_dict = {
    "End_Random": ":/images/images/MachineSetup/WindParam/End_Random.PNG",
    "End_Preform": ":/images/images/MachineSetup/WindParam/End_Preform.PNG",
    "Dim_Ext_Rotor": ":/images/images/MachineSetup/MachineDimension/Dim_ER.png",
    "Dim_In_Rotor_No_Shaft": ":/images/images/MachineSetup/MachineDimension/Dim_IRNS.png",
    "Dim_In_Rotor_Shaft": ":/images/images/MachineSetup/MachineDimension/Dim_IRS.png",
    "WindingCW2LT": ":/images/images/MachineSetup/WindingPattern/Type_Wind_1.PNG",
    "WindingCW1L": ":/images/images/MachineSetup/WindingPattern/Type_Wind_2.PNG",
    "WindingDW2L": ":/images/images/MachineSetup/WindingPattern/Type_Wind_3.PNG",
    "WindingDW1L": ":/images/images/MachineSetup/WindingPattern/Type_Wind_4.PNG",
    "WindingCW2LR": ":/images/images/MachineSetup/WindingPattern/Type_Wind_5.PNG",
    "WindingKoil": ":/images/images/MachineSetup/WindingPattern/Type_Wind_7.png",
    "WindingMat": ":/images/images/MachineSetup/WindingPattern/Type_Wind_7.png",
    "SCIM": ":/images/images/MachineSetup/SCIM.png",
    "DFIM": ":/images/images/MachineSetup/DFIM.png",
    "SyRM": ":/images/images/MachineSetup/SyRM.png",
    "SPMSM": ":/images/images/MachineSetup/SPMSM.png",
    "IPMSM": ":/images/images/MachineSetup/IPMSM.png",
    "BPMSM": ":/images/images/MachineSetup/BPMSM.png",
    "WRSM": ":/images/images/MachineSetup/WRSM.png",
}



# -*- coding: utf-8 -*-

from numpy import arcsin, cos, pi, sin


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW14
        A SlotW14 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    Rbo = self.get_Rbo()
    [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, Z9] = self._comp_point_coordinate()
    W1 = abs(Z7.imag) * 2

    S1 = (W1 + self.W0) * self.H1 / 2
    S0 = self.W0 * self.H0

    # The bottom is an arc
    alpha = self.comp_angle_opening()
    Sarc = (Rbo ** 2.0) / 2.0 * (alpha - sin(alpha))

    Swind = self.comp_surface_wind()

    if self.is_outwards():
        return S1 + S0 + Swind - Sarc
    else:
        return S1 + S0 + Swind + Sarc



# -*- coding: utf-8 -*-

from .....Functions.Plot.plot_A_surf import plot_A_surf as plot_A_surf_fct
from .....Functions.init_fig import init_fig
from SciDataTool import VectorField

from matplotlib.pyplot import subplots


def plot_A_surf(
    self,
    Data_str,
    is_deg=True,
    t_max=None,
    a_max=400,
    z_min=None,
    z_max=None,
    is_norm=False,
    unit="SI",
    colormap=None,
    save_path=None,
    component_list=None,
):
    """Plots the isosurface of a field in 3D

    Parameters
    ----------
    self : Output
        an Output object
    Data_str : str
        name of the Data Object to plot (e.g. "mag.Br")
    is_deg : bool
        boolean indicating if the angle must be converted to degrees
    t_max : float
        maximum value of the time for the x axis
    a_max : float
        maximum value of the angle for the y axis
    z_min : float
        minimum value of the amplitude for the z axis
    z_max : float
        maximum value of the amplitude for the z axis
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    colormap : colormap object
        colormap prescribed by user
    save_path : str
        path and name of the png file to save
    component_list : list
        list of component names to plot in separate figures
    """

    # Get Data object name
    phys = getattr(self, Data_str.split(".")[0])
    data = getattr(phys, Data_str.split(".")[1])

    # Call the plot function
    if isinstance(data, VectorField):
        if component_list is None:  # default: extract all components
            component_list = data.components.keys()
        ncomp = len(component_list)
        fig, axs = subplots(
            1,
            ncomp,
            tight_layout=True,
            figsize=(20, 10),
            subplot_kw=dict(projection="3d"),
        )
        for i, comp in enumerate(component_list):
            plot_A_surf_fct(
                data.components[comp],
                is_deg=is_deg,
                t_max=t_max,
                a_max=a_max,
                z_min=z_min,
                z_max=z_max,
                is_norm=is_norm,
                unit=unit,
                colormap=colormap,
                save_path=save_path,
                fig=fig,
                subplot_index=i,
            )

    else:
        (fig, axes, patch_leg, label_leg) = init_fig(
            None, shape="rectangle", is_3d=True
        )
        plot_A_surf_fct(
            data,
            is_deg=is_deg,
            t_max=t_max,
            a_max=a_max,
            z_min=z_min,
            z_max=z_max,
            is_norm=is_norm,
            unit=unit,
            colormap=colormap,
            save_path=save_path,
            fig=fig,
        )

    fig.show()



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch, Polygon
from matplotlib.pyplot import axis, legend
from numpy import array, exp, pi

from ....Functions.init_fig import init_fig
from ....definitions import config_dict

MAGNET_COLOR = config_dict["PLOT"]["COLOR_DICT"]["MAGNET_COLOR"]


def plot(self, fig=None, display_field=False):
    """Plot the Magnet in a matplotlib fig

    Parameters
    ----------
    self : Magnet
        A Magnet object
    fig :
        if None, open a new fig and plot, else add to the current
        one (Default value = None)
    display_field : bool
        if true, will display magnetization field arrow (Default value = False)

    Returns
    -------
    None
    """

    surf_list = self.build_geometry()
    patches = list()
    for surf in surf_list:
        patches.extend(surf.get_patches(color=MAGNET_COLOR))

    # Display the result
    (fig, axes, patch_leg, label_leg) = init_fig(fig)
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    axes.set_title("Magnet")

    # Add the magnet to the fig
    for patch in patches:
        axes.add_patch(patch)

    # Add the field if needed
    if display_field:
        arrow_list = self.build_magnetization_field(Rbo)
        for arrow in arrow_list:
            axes.annotate(
                "",
                xy=(arrow[0].real, arrow[0].imag),
                xycoords="data",
                xytext=(arrow[1].real, arrow[1].imag),
                textcoords="data",
                arrowprops=dict(arrowstyle="->", connectionstyle="arc3"),
            )

    # Axis Setup
    axis("equal")

    # Legend setup
    if "Magnet" not in label_leg:
        patch_leg.append(Patch(color=MAGNET_COLOR))
        label_leg.append("Magnet")
        legend(patch_leg, label_leg)
    fig.show()



from logging import Filter, DEBUG, INFO, WARNING, ERROR, CRITICAL
from .definitions import USER_DIR

"""
File containing every Pyleecan loggers.
The choice between LOGGING_CONFIG_CONSOLE and LOGGING_CONFIG_FILE is made in pyleecan.__init__.py
"""

# Default level in every loggers
DEFAULT_LEVEL = "INFO"


class LevelFilter(Filter):
    """Class to filter a unique log level"""

    def __init__(self, level):
        """
        LevelFilter constructor

        Parameters:
            level : int
                log level to keep
        """
        self.level = level

    def filter(self, record):
        return record.levelno == self.level


# LOGGING_CONFIG_CONSOLE display every logs in the console
LOGGING_CONFIG_CONSOLE = {
    "version": 1,
    # Define loggers
    "loggers": {
        "": {"level": "NOTSET", "handlers": []},  # root logger
        "Pyleecan": {
            "level": "DEBUG",
            "propagate": False,
            "handlers": [  # Different handler enable to use differents formats, and to handle different stream (stdout, stderr,...)
                "debug_console_handler",
                "info_console_handler",
                "warning_console_handler",
                "error_console_handler",
                "critical_console_handler",
            ],
        },
        "Pyleecan.Output": {"level": DEFAULT_LEVEL, "propagate": True, "handlers": []},
        "Pyleecan.Simulation": {
            "level": DEFAULT_LEVEL,
            "propagate": True,
            "handlers": [],
        },
        "Pyleecan.Machine": {"level": DEFAULT_LEVEL, "propagate": True, "handlers": []},
        "Pyleecan.OutElec": {
            "level": DEFAULT_LEVEL,
            "propagate": True,
            "handlers": [],  # Different handler enable to use differents formats, and to handle different stream (stdout, stderr,...)
        },
        "Pyleecan.OutMag": {"level": DEFAULT_LEVEL, "propagate": True, "handlers": []},
        "Pyleecan.OutStruct": {
            "level": DEFAULT_LEVEL,
            "propagate": True,
            "handlers": [],
        },
        "Pyleecan.OutGeo": {"level": DEFAULT_LEVEL, "propagate": True, "handlers": []},
        "Pyleecan.OptiSolver": {
            "level": DEFAULT_LEVEL,
            "propagate": False,
            "handlers": ["opti_file_handler"],  # <-- redirection in a file
        },
    },
    # Handlers definition
    "handlers": {
        "debug_console_handler": {
            "level": "DEBUG",
            "formatter": "default_formatter",
            "filters": ["debug_filter"],
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stdout",
        },
        "info_console_handler": {
            "level": "INFO",
            "formatter": "default_formatter",
            "filters": ["info_filter"],
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stdout",
        },
        "warning_console_handler": {
            "level": "WARNING",
            "formatter": "default_formatter",
            "filters": ["warning_filter"],
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stdout",
        },
        "error_console_handler": {
            "level": "ERROR",
            "formatter": "default_formatter",
            "filters": ["error_filter"],
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stderr",
        },
        "critical_console_handler": {
            "level": "CRITICAL",
            "formatter": "default_formatter",
            "filters": ["critical_filter"],
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stderr",
        },
        "opti_file_handler": {  # <------- FileHandler
            "level": "DEBUG",
            "formatter": "file_formatter",
            "class": "logging.FileHandler",
            "filename": USER_DIR + "/Pyleecan_optimization.log",  # log file
            "mode": "a",
        },
        #         'info_rotating_file_handler': {
        #             'level': 'INFO',
        #             'formatter': 'info',
        #             'class': 'logging.handlers.RotatingFileHandler',
        #             'filename': 'info.log',
        #             'mode': 'a',
        #             'maxBytes': 1048576,
        #             'backupCount': 10
        #         },
    },
    # Define filters
    "filters": {
        "debug_filter": {"()": LevelFilter, "level": DEBUG},
        "info_filter": {"()": LevelFilter, "level": INFO},
        "warning_filter": {"()": LevelFilter, "level": WARNING},
        "error_filter": {"()": LevelFilter, "level": ERROR},
        "critical_filter": {"()": LevelFilter, "level": CRITICAL},
    },
    # Define formatters
    "formatters": {
        "default_formatter": {
            "format": "%(asctime)s-%(levelname)s-%(name)s: %(message)s"
        },
        "file_formatter": {"format": "%(asctime)s-%(levelname)s-%(name)s: %(message)s"},
    },
}

# LOGGING_CONFIG_CONSOLE display every logs in the console
LOGGING_CONFIG_FILE = {
    "version": 1,
    # Define loggers
    "loggers": {
        "": {"level": "NOTSET", "handlers": []},  # root logger
        "Pyleecan": {
            "level": DEFAULT_LEVEL,
            "propagate": False,
            "handlers": ["Pyleecan_fh"],
        },
        "Pyleecan.Output": {
            "level": DEFAULT_LEVEL,
            "propagate": False,
            "handlers": ["Output_fh"],
        },
        "Pyleecan.Simulation": {
            "level": DEFAULT_LEVEL,
            "propagate": False,
            "handlers": ["Simulation_fh"],
        },
        "Pyleecan.Machine": {
            "level": DEFAULT_LEVEL,
            "propagate": False,
            "handlers": ["Machine_fh"],
        },
        "Pyleecan.OutElec": {
            "level": DEFAULT_LEVEL,
            "propagate": False,
            "handlers": ["OutElec_fh"],
        },
        "Pyleecan.OutMag": {"level": DEFAULT_LEVEL, "propagate": False, "handlers": []},
        "Pyleecan.OutStruct": {
            "level": DEFAULT_LEVEL,
            "propagate": False,
            "handlers": ["OutStruct_fh"],
        },
        "Pyleecan.OutGeo": {
            "level": DEFAULT_LEVEL,
            "propagate": False,
            "handlers": ["OutGeo_fh"],
        },
        "Pyleecan.OptiSolver": {
            "level": DEFAULT_LEVEL,
            "propagate": False,
            "handlers": ["opti_file_handler"],  # <-- redirection in a file
        },
    },
    # Handlers definition
    "handlers": {
        #         'info_rotating_file_handler': {
        #             'level': 'INFO',
        #             'formatter': 'info',
        #             'class': 'logging.handlers.RotatingFileHandler',
        #             'filename': 'info.log',
        #             'mode': 'a',
        #             'maxBytes': 1048576,
        #             'backupCount': 10
        #         },
        "Pyleecan_fh": {
            "level": "DEBUG",
            "formatter": "file_formatter",
            "class": "logging.FileHandler",
            "filename": USER_DIR + "/Pyleecan_optimization.log",
            "mode": "a",
        },
        "Output_fh": {
            "level": "DEBUG",
            "formatter": "file_formatter",
            "class": "logging.FileHandler",
            "filename": USER_DIR + "/Pyleecan_optimization.log",
            "mode": "a",
        },
        "Simulation_fh": {
            "level": "DEBUG",
            "formatter": "file_formatter",
            "class": "logging.FileHandler",
            "filename": USER_DIR + "/Pyleecan_optimization.log",
            "mode": "a",
        },
        "OutElec_fh": {
            "level": "DEBUG",
            "formatter": "file_formatter",
            "class": "logging.FileHandler",
            "filename": USER_DIR + "/Pyleecan_optimization.log",
            "mode": "a",
        },
        "OutMag_fh": {
            "level": "DEBUG",
            "formatter": "file_formatter",
            "class": "logging.FileHandler",
            "filename": USER_DIR + "/Pyleecan_optimization.log",
            "mode": "a",
        },
        "OutStruct_fh": {
            "level": "DEBUG",
            "formatter": "file_formatter",
            "class": "logging.FileHandler",
            "filename": USER_DIR + "/Pyleecan_optimization.log",
            "mode": "a",
        },
        "Machine_fh": {
            "level": "DEBUG",
            "formatter": "file_formatter",
            "class": "logging.FileHandler",
            "filename": USER_DIR + "/Pyleecan_optimization.log",
            "mode": "a",
        },
        "Opti_fh": {
            "level": "DEBUG",
            "formatter": "file_formatter",
            "class": "logging.FileHandler",
            "filename": USER_DIR + "/Pyleecan_optimization.log",
            "mode": "a",
        },
        #         'info_rotating_file_handler': {
        #             'level': 'INFO',
        #             'formatter': 'info',
        #             'class': 'logging.handlers.RotatingFileHandler',
        #             'filename': 'info.log',
        #             'mode': 'a',
        #             'maxBytes': 1048576,
        #             'backupCount': 10
        #         },
    },
    # Define filters
    "filters": {
        "debug_filter": {"()": LevelFilter, "level": DEBUG},
        "info_filter": {"()": LevelFilter, "level": INFO},
        "warning_filter": {"()": LevelFilter, "level": WARNING},
        "error_filter": {"()": LevelFilter, "level": ERROR},
        "critical_filter": {"()": LevelFilter, "level": CRITICAL},
    },
    # Define formatters
    "formatters": {
        "default_formatter": {
            "format": "%(asctime)s-%(levelname)s-%(name)s: %(message)s"
        },
        "file_formatter": {"format": "%(asctime)s-%(levelname)s-%(name)s: %(message)s"},
    },
}



import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import LinearSegmentedColormap


def plot_generation(self, x_symbol, y_symbol, ax=None):
    """Plot every fitness values according to the two fitness

    Parameters
    ----------
    self : XOutput
    obj1 : str
        symbol of the ParamExplorer or the DataKeeper
    obj2 : str
        symbol of the ParamExplorer or the DataKeeper
    """

    # TODO define the colormap according to Pyleecan graphical chart
    # Colormap definition
    cm = LinearSegmentedColormap.from_list(
        "colormap",
        [(35 / 255, 89 / 255, 133 / 255), (250 / 255, 202 / 255, 56 / 255)],
        N=max(self["ngen"].result) + 1,
    )

    # Get fitness and ngen
    is_valid = np.array(self["is_valid"].result)
    ngen = np.array(self["ngen"].result)

    # Keep only valid values
    indx = np.where(is_valid)[0]

    ngen = ngen[indx]

    # Get x_data
    if x_symbol in self.keys():  # DataKeeper
        x_data = self[x_symbol]
        x_values = np.array(x_data.result)[indx]
    else:  # ParamSetter
        x_data = self.get_paramexplorer(x_symbol)
        x_values = np.array(x_data.value)[indx]

    # x_label definition
    x_label = x_symbol
    if x_data.unit not in ["", None]:
        x_label += " [{}]".format(x_data.unit)

    # Get y_data
    if y_symbol in self.keys():  # DataKeeper
        y_data = self[y_symbol]
        y_values = np.array(y_data.result)[indx]
    else:  # ParamSetter
        y_data = self.get_paramexplorer(y_symbol)
        y_values = np.array(y_data.value)[indx]

    # y_label definition
    y_label = y_symbol
    if y_data.unit not in ["", None]:
        y_label += " [{}]".format(y_data.unit)

    if ax is None:
        fig, ax = plt.subplots()

        # Plot fitness values
        scatter = ax.scatter(x_values, y_values, s=8, c=ngen, cmap=cm)

        # Add legend
        legend1 = ax.legend(
            *scatter.legend_elements(), loc="upper right", title="Generation"
        )
        ax.add_artist(legend1)

        # Extend xlim to give some space to the legend
        left, right = ax.get_xlim()
        ax.set_xlim(left, right + 0.2 * abs(right - left))

        ax.set_xlabel(x_label)
        ax.set_ylabel(y_label)
        ax.set_title("Fitness values for each individual")

        fig.show()

    else:
        # Plot fitness values
        scatter = ax.scatter(x_values, y_values, s=8, c=ngen, cmap=cm)

        # Add legend
        legend1 = ax.legend(
            *scatter.legend_elements(), loc="upper right", title="Generation"
        )
        ax.add_artist(legend1)

        # Extend xlim to give some space to the legend
        left, right = ax.get_xlim()
        ax.set_xlim(left, right + 0.2 * abs(right - left))

        ax.set_xlabel(x_label)
        ax.set_ylabel(y_label)
        ax.set_title("Fitness values for each individual")

        return ax



# -*- coding: utf-8 -*-

from numpy import sin


def comp_W0m(self):
    """Compute (or return) W0 in [m]

    Parameters
    ----------
    self : SlotMFlat
        A SlotMFlat object

    Returns
    -------
    W0m: float
        W0 in [m]

    """

    Rbo = self.get_Rbo()
    if self.W0_is_rad:  # Convert W0 to m
        return 2 * Rbo * sin((self.W0 / 2))
    else:
        return self.W0



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/GUI_Option/GUIOption.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/GUIOption/GUIOption
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError
from .Unit import Unit


class GUIOption(FrozenClass):

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, unit=-1, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.unit = unit

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        GUIOption_str = ""
        if self.parent is None:
            GUIOption_str += "parent = None " + linesep
        else:
            GUIOption_str += "parent = " + str(type(self.parent)) + " object" + linesep
        if self.unit is not None:
            tmp = self.unit.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            GUIOption_str += "unit = " + tmp
        else:
            GUIOption_str += "unit = None" + linesep + linesep
        return GUIOption_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.unit != self.unit:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        GUIOption_dict = dict()
        if self.unit is None:
            GUIOption_dict["unit"] = None
        else:
            GUIOption_dict["unit"] = self.unit.as_dict()
        # The class name is added to the dict for deserialisation purpose
        GUIOption_dict["__class__"] = "GUIOption"
        return GUIOption_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.unit is not None:
            self.unit._set_None()

    def _get_unit(self):
        """getter of unit"""
        return self._unit

    def _set_unit(self, value):
        """setter of unit"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "unit")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Unit()
        check_var("unit", value, "Unit")
        self._unit = value

        if self._unit is not None:
            self._unit.parent = self

    unit = property(
        fget=_get_unit,
        fset=_set_unit,
        doc=u"""Unit options

        :Type: Unit
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/MachineSyRM.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/MachineSyRM
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .MachineSync import MachineSync

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.MachineSyRM.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Machine.MachineSyRM.get_lam_list import get_lam_list
except ImportError as error:
    get_lam_list = error

try:
    from ..Methods.Machine.MachineSyRM.get_machine_type import get_machine_type
except ImportError as error:
    get_machine_type = error


from ._check import InitUnKnowClassError
from .LamHole import LamHole
from .LamSlotWind import LamSlotWind
from .Frame import Frame
from .Shaft import Shaft


class MachineSyRM(MachineSync):
    """Synchronous Reluctance Machine"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Machine.MachineSyRM.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use MachineSyRM method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Machine.MachineSyRM.get_lam_list
    if isinstance(get_lam_list, ImportError):
        get_lam_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MachineSyRM method get_lam_list: " + str(get_lam_list)
                )
            )
        )
    else:
        get_lam_list = get_lam_list
    # cf Methods.Machine.MachineSyRM.get_machine_type
    if isinstance(get_machine_type, ImportError):
        get_machine_type = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MachineSyRM method get_machine_type: "
                    + str(get_machine_type)
                )
            )
        )
    else:
        get_machine_type = get_machine_type
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        rotor=-1,
        stator=-1,
        frame=-1,
        shaft=-1,
        name="default_machine",
        desc="",
        type_machine=1,
        logger_name="Pyleecan.Machine",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "rotor" in list(init_dict.keys()):
                rotor = init_dict["rotor"]
            if "stator" in list(init_dict.keys()):
                stator = init_dict["stator"]
            if "frame" in list(init_dict.keys()):
                frame = init_dict["frame"]
            if "shaft" in list(init_dict.keys()):
                shaft = init_dict["shaft"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "type_machine" in list(init_dict.keys()):
                type_machine = init_dict["type_machine"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.rotor = rotor
        self.stator = stator
        # Call MachineSync init
        super(MachineSyRM, self).__init__(
            frame=frame,
            shaft=shaft,
            name=name,
            desc=desc,
            type_machine=type_machine,
            logger_name=logger_name,
        )
        # The class is frozen (in MachineSync init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        MachineSyRM_str = ""
        # Get the properties inherited from MachineSync
        MachineSyRM_str += super(MachineSyRM, self).__str__()
        if self.rotor is not None:
            tmp = self.rotor.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            MachineSyRM_str += "rotor = " + tmp
        else:
            MachineSyRM_str += "rotor = None" + linesep + linesep
        if self.stator is not None:
            tmp = self.stator.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            MachineSyRM_str += "stator = " + tmp
        else:
            MachineSyRM_str += "stator = None" + linesep + linesep
        return MachineSyRM_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from MachineSync
        if not super(MachineSyRM, self).__eq__(other):
            return False
        if other.rotor != self.rotor:
            return False
        if other.stator != self.stator:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        # Get the properties inherited from MachineSync
        MachineSyRM_dict = super(MachineSyRM, self).as_dict()
        if self.rotor is None:
            MachineSyRM_dict["rotor"] = None
        else:
            MachineSyRM_dict["rotor"] = self.rotor.as_dict()
        if self.stator is None:
            MachineSyRM_dict["stator"] = None
        else:
            MachineSyRM_dict["stator"] = self.stator.as_dict()
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        MachineSyRM_dict["__class__"] = "MachineSyRM"
        return MachineSyRM_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.rotor is not None:
            self.rotor._set_None()
        if self.stator is not None:
            self.stator._set_None()
        # Set to None the properties inherited from MachineSync
        super(MachineSyRM, self)._set_None()

    def _get_rotor(self):
        """getter of rotor"""
        return self._rotor

    def _set_rotor(self, value):
        """setter of rotor"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "rotor"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = LamHole()
        check_var("rotor", value, "LamHole")
        self._rotor = value

        if self._rotor is not None:
            self._rotor.parent = self

    rotor = property(
        fget=_get_rotor,
        fset=_set_rotor,
        doc=u"""Machine's Rotor

        :Type: LamHole
        """,
    )

    def _get_stator(self):
        """getter of stator"""
        return self._stator

    def _set_stator(self, value):
        """setter of stator"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "stator"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = LamSlotWind()
        check_var("stator", value, "LamSlotWind")
        self._stator = value

        if self._stator is not None:
            self._stator.parent = self

    stator = property(
        fget=_get_stator,
        fset=_set_stator,
        doc=u"""Machine's Stator

        :Type: LamSlotWind
        """,
    )



# -*- coding: utf-8 -*-

from ....Classes.MagFEMM import MagFEMM


def set_reused_data(self, simu, output):
    """Resuse some data from the reference simulation to skip computation

    Parameters
    ----------
    self : VarSimu
        a VarSimu object
    simu : Simulation
        The simulation to update
    output : Output
        Output from the reference simulation to enforce
    """

    if self.is_reuse_femm_file and isinstance(simu.mag, MagFEMM):
        simu.mag.import_file = output.mag.FEMM_dict["path_save"]
        simu.mag.FEMM_dict = output.mag.FEMM_dict



# -*- coding: utf-8 -*-

from numpy import array, exp, arctan

from ....Classes.SurfLine import SurfLine
from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Segment, Arc1) needed to plot the Magnet.
    The list represents a closed surface.
    The ending point of a curve is always the starting point of the next curve
    in the list

    Parameters
    ----------
    self : Magnet_Type_11
        A Magnet_Type_11 object
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_simplified: bool
        True to avoid line superposition

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the magnet

    """

    # defining label for type_magnetization
    if self.type_magnetization == 0:
        t_p = "Radial"
    elif self.type_magnetization == 1:
        t_p = "Parallel"
    elif self.type_magnetization == 2:
        t_p = "Hallbach"

    [Z1, Z2, Z3, Zs3, Zs4, Z4, Zref] = self._comp_point_coordinate()
    # Creation of curve
    curve_list = list()
    if is_simplified and W0 > self.Wmag:
        curve_list.append(Segment(Z1, Z3))
    elif is_simplified and H0 < self.Hmag:
        curve_list.append(Segment(Zs3, Z3))
    elif not is_simplified:
        curve_list.append(Segment(Z1, Z3))

    curve_list.append(Arc1(Z3, Z4, abs(Z3)))

    if is_simplified and W0 > self.Wmag:
        curve_list.append(Segment(Z4, Z2))
    elif is_simplified and H0 < self.Hmag:
        curve_list.append(Segment(Z4, Zs4))
    elif not is_simplified:
        curve_list.append(Segment(Z4, Z2))

    if not is_simplified:
        curve_list.append(Arc1(Z2, Z1, -abs(Z2), is_trigo_direction=False))

    surf_list = list()
    surf_list.append(
        SurfLine(
            line_list=curve_list,
            label="MagnetRotor" + t_p + "_N_R0_T0_S0",
            point_ref=Zref,
        )
    )

    # Apply transformation
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Mesh/Solution.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Mesh/Solution
"""

from os import linesep
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError


class Solution(FrozenClass):
    """Abstract class for solution related classes."""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        type_cell="triangle",
        label=None,
        dimension=2,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "type_cell" in list(init_dict.keys()):
                type_cell = init_dict["type_cell"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
            if "dimension" in list(init_dict.keys()):
                dimension = init_dict["dimension"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.type_cell = type_cell
        self.label = label
        self.dimension = dimension

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Solution_str = ""
        if self.parent is None:
            Solution_str += "parent = None " + linesep
        else:
            Solution_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Solution_str += 'type_cell = "' + str(self.type_cell) + '"' + linesep
        Solution_str += 'label = "' + str(self.label) + '"' + linesep
        Solution_str += "dimension = " + str(self.dimension) + linesep
        return Solution_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.type_cell != self.type_cell:
            return False
        if other.label != self.label:
            return False
        if other.dimension != self.dimension:
            return False
        return True

    def as_dict(self):
        """Convert this object in a json seriable dict (can be use in __init__)"""

        Solution_dict = dict()
        Solution_dict["type_cell"] = self.type_cell
        Solution_dict["label"] = self.label
        Solution_dict["dimension"] = self.dimension
        # The class name is added to the dict for deserialisation purpose
        Solution_dict["__class__"] = "Solution"
        return Solution_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.type_cell = None
        self.label = None
        self.dimension = None

    def _get_type_cell(self):
        """getter of type_cell"""
        return self._type_cell

    def _set_type_cell(self, value):
        """setter of type_cell"""
        check_var("type_cell", value, "str")
        self._type_cell = value

    type_cell = property(
        fget=_get_type_cell,
        fset=_set_type_cell,
        doc=u"""Type of cell (Point, Segment2, Triangle3, etc.)

        :Type: str
        """,
    )

    def _get_label(self):
        """getter of label"""
        return self._label

    def _set_label(self, value):
        """setter of label"""
        check_var("label", value, "str")
        self._label = value

    label = property(
        fget=_get_label,
        fset=_set_label,
        doc=u"""Label to identify the solution

        :Type: str
        """,
    )

    def _get_dimension(self):
        """getter of dimension"""
        return self._dimension

    def _set_dimension(self, value):
        """setter of dimension"""
        check_var("dimension", value, "int", Vmin=1, Vmax=3)
        self._dimension = value

    dimension = property(
        fget=_get_dimension,
        fset=_set_dimension,
        doc=u"""Dimension of the physical problem

        :Type: int
        :min: 1
        :max: 3
        """,
    )



# -*- coding: utf-8 -*-

from numpy import angle
from scipy.optimize import fsolve

from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine


def build_geometry_wind(self, Nrad, Ntan, is_simplified=False, alpha=0, delta=0):
    """Split the slot winding area in several zone

    Parameters
    ----------
    self : SlotW14
        A SlotW14 object
    Nrad : int
        Number of radial layer
    Ntan : int
        Number of tangentiel layer
    is_simplified : bool
        boolean to specify if coincident lines are considered as one or different lines (Default value = False)
    alpha : float
        Angle for rotation (Default value = 0) [rad]
    delta : Complex
        complex for translation (Default value = 0)

    Returns
    -------
    surf_list:
        List of surface delimiting the winding zone

    """
    if self.get_is_stator():  # check if the slot is on the stator
        st = "S"
    else:
        st = "R"

    [Z9, Z8, Z7, Z6, Z5, Z4, Z3, Z2, Z1] = self._comp_point_coordinate()

    Ztan1 = (Z3 + Z7) / 2.0
    Ztan2 = Z5
    Zmid = (Ztan1 + Ztan2) / 2.0

    R5 = Z5.real
    R6 = Z6.real
    R7 = Z7.real

    if abs(R5 - R6) < abs(R6 - R7):  # Zrad1 between Z6 and Z7
        x = fsolve(lambda x: angle((Z7 - (Zmid + 1j * x)) / (Z7 - Z6)), Z6.imag)
        Zrad1 = Zmid + 1j * x[0]
        Zrad2 = Zrad1.conjugate()
    elif abs(R5 - R6) > abs(R6 - R7):  # Zrad1 between Z6 and Z5
        x = fsolve(lambda x: angle((Z6 - (Zmid + 1j * x)) / (Z6 - Z5)), Z6.imag)
        Zrad1 = Zmid + 1j * x[0]
        Zrad2 = Zrad1.conjugate()
    else:
        Zrad1 = Z6
        Zrad2 = Z4

    surf_list = list()
    if Nrad == 1 and Ntan == 2:
        if is_simplified:
            # Part 1 (0,0)
            surf_list.append(
                gen_curve_list_simplified([Z7, Ztan1, Ztan2], "0", "0", st)
            )
            # Part 2 (0,1)
            surf_list.append(gen_curve_list_simplified([Ztan1, Z3], "0", "1", st))
        else:
            # Part 1 (0,0)
            surf_list.append(gen_curve_list([Z7, Ztan1, Ztan2, Z6], "0", "0", st))
            # Part 2 (0,1)
            surf_list.append(gen_curve_list([Ztan1, Z3, Z4, Ztan2], "0", "1", st))
    elif Nrad == 2 and Ntan == 1:
        if is_simplified:
            # Part 1 (0,0)
            if abs(R5 - R6) > abs(R6 - R7):  # Zrad1 between Z6 and Z5
                point_list = [Z7, Z3, Z4, Zrad2, Zrad1, Z6]
            else:
                point_list = [Z7, Z3, Zrad2, Zrad1]
            line1 = Segment(Z7, Z3)
            line2 = Segment(Zrad2, Zrad1)
            point_ref = 0
            for Z in point_list:
                point_ref += Z
            surface = SurfLine(
                line_list=[line1, line2],
                label="Wind" + st + "_R0_T0_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part 2 (1,0)
            if abs(R5 - R6) < abs(R6 - R7):  # Zrad1 between Z6 and Z7
                point_list = [Zrad1, Zrad2, Z4, Z5, Z6]
            else:
                point_list = [Zrad1, Zrad2, Z5]
            point_ref = 0
            for Z in point_list:
                point_ref += Z

            point_ref = point_ref / len(point_list)  # point reference of the surface
            surface = SurfLine(
                line_list=[], label="Wind" + st + "_R1_T0_S0", point_ref=point_ref
            )
            surf_list.append(surface)
        else:

            # Part 1 (0,0)
            if abs(R5 - R6) > abs(R6 - R7):  # Zrad1 between Z6 and Z5
                point_list = [Z7, Z3, Z4, Zrad2, Zrad1, Z6]
            else:
                point_list = [Z7, Z3, Zrad2, Zrad1]
            surf_list.append(gen_curve_list(point_list, "0", "0", st))
            # Part 2 (1,0)
            if abs(R5 - R6) < abs(R6 - R7):  # Zrad1 between Z6 and Z7
                point_list = [Zrad1, Zrad2, Z4, Z5, Z6]
            else:
                point_list = [Zrad1, Zrad2, Z5]

            surf_list.append(gen_curve_list(point_list, "1", "0", st))
    elif Nrad == 2 and Ntan == 2:
        if is_simplified:
            # Part 1 (0,0)
            point_list = [Z7, Ztan1, Zmid, Zrad1]
            surf_list.append(gen_curve_list_simplified(point_list, "0", "0", st))
            # Part 2 (1,0)
            point_list = [Zmid, Ztan2]
            surf_list.append(gen_curve_list_simplified(point_list, "1", "0", st))
            # Part 3 (0,1)
            if abs(R5 - R6) < abs(R6 - R7):  # Zrad1 between Z6 and Z7
                point_list = [Ztan1, Z3, Zrad2, Zmid]
            else:
                point_list = [Ztan1, Z3, Z4, Zrad2, Zmid]

            line1 = Segment(Ztan1, Z3)
            line2 = Segment(Zrad2, Zmid)
            point_ref = 0
            for Z in point_list:
                point_ref += Z

            point_ref = point_ref / len(point_list)  # point reference of the surface
            surface = SurfLine(
                line_list=[line1, line2],
                label="Wind" + st + "_R0_T1_S0",
                point_ref=point_ref,
            )
            surf_list.append(surface)
            # Part 4 (1,1)
            if abs(R5 - R6) > abs(R6 - R7):  # Zrad1 between Z6 and Z5
                point_list = [Zmid, Zrad2, Z5]
            else:
                point_list = [Zmid, Zrad2, Z4, Ztan2]

            point_ref = 0
            for Z in point_list:
                point_ref += Z

            point_ref = point_ref / len(point_list)  # point reference of the surface
            surface = SurfLine(
                line_list=[], label="Wind" + st + "_R1_T1_S0", point_ref=point_ref
            )
            surf_list.append(surface)
        else:

            # Part 1 (0,0)
            if abs(R5 - R6) < abs(R6 - R7):  # Zrad1 between Z6 and Z7
                point_list = [Z7, Ztan1, Zmid, Zrad1]
            else:
                point_list = [Z7, Ztan1, Zmid, Zrad1, Z6]
            surf_list.append(gen_curve_list(point_list, "0", "0", st))
            # Part 2 (1,0)
            if abs(R5 - R6) > abs(R6 - R7):  # Zrad1 between Z6 and Z5
                point_list = [Zrad1, Zmid, Ztan2]
            else:
                point_list = [Zrad1, Zmid, Ztan2, Z6]

            surf_list.append(gen_curve_list(point_list, "1", "0", st))
            # Part 3 (0,1)
            if abs(R5 - R6) < abs(R6 - R7):  # Zrad1 between Z6 and Z7
                point_list = [Ztan1, Z3, Zrad2, Zmid]
            else:
                point_list = [Ztan1, Z3, Z4, Zrad2, Zmid]
            surf_list.append(gen_curve_list(point_list, "0", "1", st))
            # Part 4 (1,1)
            if abs(R5 - R6) > abs(R6 - R7):  # Zrad1 between Z6 and Z5
                point_list = [Zmid, Zrad2, Z5]
            else:
                point_list = [Zmid, Zrad2, Z4, Ztan2]

            surf_list.append(gen_curve_list(point_list, "1", "1", st))
    else:
        surf_list.append(gen_curve_list([Z7, Z3, Z4, Z5, Z6], "0", "0", st))

    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)
    return surf_list


def gen_curve_list(point_list, RX, TX, st):
    """

    Parameters
    ----------
    point_list :

    RX :

    TX :

    st :


    Returns
    -------

    """
    curve_list = list()
    for ii in range(len(point_list) - 1):
        curve_list.append(Segment(point_list[ii], point_list[ii + 1]))
    curve_list.append(Segment(point_list[-1], point_list[0]))
    res = 0
    for Z in point_list:
        res += Z
    point_ref = res / len(point_list)
    surface = SurfLine(
        line_list=curve_list,
        label="Wind" + st + "_R" + RX + "_T" + TX + "_S0",
        point_ref=point_ref,
    )
    return surface


def gen_curve_list_simplified(point_list, RX, TX, st):
    """

    Parameters
    ----------
    point_list :

    RX :

    TX :

    st :


    Returns
    -------

    """
    curve_list = list()
    for ii in range(len(point_list) - 1):
        curve_list.append(Segment(point_list[ii], point_list[ii + 1]))
    res = 0
    for Z in point_list:
        res += Z
    point_ref = res / len(point_list)
    surface = SurfLine(
        line_list=curve_list,
        label="Wind" + st + "_R" + RX + "_T" + TX + "_S0",
        point_ref=point_ref,
    )
    return surface



# -*- coding: utf-8 -*-

# File generated according to WImportSin.ui
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_WImportExcel(object):
    def setupUi(self, WImportExcel):
        WImportExcel.setObjectName("WImportExcel")
        WImportExcel.resize(546, 511)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(WImportExcel.sizePolicy().hasHeightForWidth())
        WImportExcel.setSizePolicy(sizePolicy)
        WImportExcel.setMinimumSize(QtCore.QSize(0, 0))
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(WImportExcel)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.w_file_path = WPathSelector(WImportExcel)
        self.w_file_path.setObjectName("w_file_path")
        self.verticalLayout_3.addWidget(self.w_file_path)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.in_sheet = QtWidgets.QLabel(WImportExcel)
        self.in_sheet.setObjectName("in_sheet")
        self.horizontalLayout_2.addWidget(self.in_sheet)
        self.c_sheet = QtWidgets.QComboBox(WImportExcel)
        self.c_sheet.setObjectName("c_sheet")
        self.horizontalLayout_2.addWidget(self.c_sheet)
        self.in_range = QtWidgets.QLabel(WImportExcel)
        self.in_range.setObjectName("in_range")
        self.horizontalLayout_2.addWidget(self.in_range)
        self.le_range = QtWidgets.QLineEdit(WImportExcel)
        self.le_range.setObjectName("le_range")
        self.horizontalLayout_2.addWidget(self.le_range)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.g_axe1 = QtWidgets.QGroupBox(WImportExcel)
        self.g_axe1.setObjectName("g_axe1")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.g_axe1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.c_axe1_name = QtWidgets.QComboBox(self.g_axe1)
        self.c_axe1_name.setObjectName("c_axe1_name")
        self.horizontalLayout_3.addWidget(self.c_axe1_name)
        self.c_axe1_type = QtWidgets.QComboBox(self.g_axe1)
        self.c_axe1_type.setObjectName("c_axe1_type")
        self.horizontalLayout_3.addWidget(self.c_axe1_type)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.w_axe1 = QtWidgets.QWidget(self.g_axe1)
        self.w_axe1.setObjectName("w_axe1")
        self.verticalLayout.addWidget(self.w_axe1)
        self.verticalLayout_3.addWidget(self.g_axe1)
        self.g_axe2 = QtWidgets.QGroupBox(WImportExcel)
        self.g_axe2.setObjectName("g_axe2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.g_axe2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.c_axe2_name = QtWidgets.QComboBox(self.g_axe2)
        self.c_axe2_name.setObjectName("c_axe2_name")
        self.horizontalLayout_4.addWidget(self.c_axe2_name)
        self.c_axe2_type = QtWidgets.QComboBox(self.g_axe2)
        self.c_axe2_type.setObjectName("c_axe2_type")
        self.horizontalLayout_4.addWidget(self.c_axe2_type)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.w_axe2 = QtWidgets.QWidget(self.g_axe2)
        self.w_axe2.setObjectName("w_axe2")
        self.verticalLayout_2.addWidget(self.w_axe2)
        self.verticalLayout_3.addWidget(self.g_axe2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(
            40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum
        )
        self.horizontalLayout.addItem(spacerItem)
        self.b_convert = QtWidgets.QPushButton(WImportExcel)
        self.b_convert.setObjectName("b_convert")
        self.horizontalLayout.addWidget(self.b_convert)
        self.b_tab = QtWidgets.QPushButton(WImportExcel)
        self.b_tab.setObjectName("b_tab")
        self.horizontalLayout.addWidget(self.b_tab)
        self.b_plot = QtWidgets.QPushButton(WImportExcel)
        self.b_plot.setObjectName("b_plot")
        self.horizontalLayout.addWidget(self.b_plot)
        self.verticalLayout_3.addLayout(self.horizontalLayout)

        self.retranslateUi(WImportExcel)
        QtCore.QMetaObject.connectSlotsByName(WImportExcel)

    def retranslateUi(self, WImportExcel):
        _translate = QtCore.QCoreApplication.translate
        WImportExcel.setWindowTitle(_translate("WImportExcel", "Form"))
        self.in_sheet.setText(_translate("WImportExcel", "Sheet: "))
        self.in_range.setText(_translate("WImportExcel", "Range: "))
        self.g_axe1.setTitle(_translate("WImportExcel", "First axe"))
        self.g_axe2.setTitle(_translate("WImportExcel", "Second axe"))
        self.b_convert.setText(_translate("WImportExcel", "Convert to Matrix"))
        self.b_tab.setText(_translate("WImportExcel", "Preview Table"))
        self.b_plot.setText(_translate("WImportExcel", "Preview Plot"))


from .....GUI.Tools.WPathSelector.WPathSelector import WPathSelector
from pyleecan.GUI.Resources import pyleecan_rc



# -*- coding: utf-8 -*-

from .....Functions.Plot.plot_A_cfft2 import plot_A_cfft2 as plot_A_cfft2_fct
from .....Functions.init_fig import init_fig
from SciDataTool import VectorField

from matplotlib.pyplot import subplots


def plot_A_cfft2(
    self,
    Data_str,
    is_elecorder=False,
    is_spaceorder=False,
    freq_max=20000,
    r_max=100,
    mag_max=None,
    N_stem=100,
    disp_negative=False,
    is_norm=False,
    unit="SI",
    save_path=None,
    component_list=None,
):
    """3D stem plot of the 2D Fourier Transform of a field

    Parameters
    ----------
    self : Output
        an Output object
    Data_str : str
        name of the Data Object to plot (e.g. "mag.Br")
    is_elecorder : bool
        boolean indicating if we want to use the electrical order for the fft axis
    is_spaceorder : bool
        boolean indicating if we want to use the spatial order for the fft axis
    freq_max : int
        maximum value of the frequency for the fft axis
    r_max : int
        maximum value of the wavenumber for the fft axis
    mag_max : int
        maximum value of the magnitude
    N_stem : int
        number of stems to plot
    disp_negative : bool
        plot negative frequencies
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    save_path : str
        path and name of the png file to save
    component_list : list
        list of component names to plot in separate figures
    """

    # Get Data object names
    phys = getattr(self, Data_str.split(".")[0])
    data = getattr(phys, Data_str.split(".")[1])

    # Call the plot function
    if isinstance(data, VectorField):
        if component_list is None:  # default: extract all components
            component_list = data.components.keys()
        ncomp = len(component_list)
        fig, axs = subplots(
            1,
            ncomp,
            tight_layout=True,
            figsize=(20, 10),
            subplot_kw=dict(projection="3d"),
        )
        for i, comp in enumerate(component_list):
            plot_A_cfft2_fct(
                data.components[comp],
                is_elecorder=is_elecorder,
                is_spaceorder=is_spaceorder,
                freq_max=freq_max,
                r_max=r_max,
                mag_max=mag_max,
                N_stem=N_stem,
                is_norm=is_norm,
                unit=unit,
                save_path=save_path,
                fig=fig,
                subplot_index=i,
            )

    else:
        (fig, axes, patch_leg, label_leg) = init_fig(
            None, shape="rectangle", is_3d=True
        )
        plot_A_cfft2_fct(
            data,
            is_elecorder=is_elecorder,
            is_spaceorder=is_spaceorder,
            freq_max=freq_max,
            r_max=r_max,
            mag_max=mag_max,
            N_stem=N_stem,
            disp_negative=disp_negative,
            is_norm=is_norm,
            unit=unit,
            save_path=save_path,
            fig=fig,
        )

    fig.show()



# -*- coding: utf-8 -*-
from .loggers import LOGGING_CONFIG_CONSOLE, LOGGING_CONFIG_FILE
from logging.config import dictConfig
from os.path import isdir


dictConfig(LOGGING_CONFIG_CONSOLE)

__version__ = "1.0.1"



from ....Methods.Slot.Slot.check import SlotCheckError


def check(self):
    """Check that the SlotCirc object is correct

    Parameters
    ----------
    self : SlotCirc
        A SlotCirc object

    Returns
    -------
    None
    """
    if self.H0 < self.W0 / 2:
        raise SC_WHCheckError("You must have W0/2 <= H0")


class SC_WHCheckError(SlotCheckError):
    """Raised when a SlotCirc has self.H0 < self.W0/2"""

    pass
